﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using SIDRASolutions.SI.API;

namespace CSharpSample
{
    class Program
    {
        const int MAXOrientation = 8; // Leg or Movement orientation is from 0 to 7; 0 = South, 1 = South East, 2 = East ...
        const int MAXMcClass = 12; // Movement Class is from 1 to 12; 1 = Light Vehicles, 2 = Heavy Vehicles, 3 = Buses ...
        const int MAXPedCode = 4; // Pedestrian Movement Code is from 0 to 3, it is for saving Leg Ped Mov data in an array; 0 = Full Crossing, 1 = Slip Lane, 2 = Staged1, 3 = Stage2. The Diagnoal Ped Mov is not in the array because one Site only has one Diagonal Ped Mov.
        const int MAXPedType = 4; // Pedestrian Movement Type is from 1 to 4; 1 = Full Crossing, 2 = Slip Lane, 3 = Staged, 4 = Diagonal.
        const int MAXPedStage = 2; // Pedestrian Movement Stage is from 1 to 2;

        static SIAPI _api = null;

        [STAThread]
        static void Main(string[] args)
        {
            // A sample to create two Signals Sites and build a Freeway Diamond Interchange Network
            CreateFreewayDiamondInterchangeNetwork();

            Console.WriteLine();

            // A sample to read Lane Disciplines data
            ReadLaneDisciplines();

            Console.WriteLine();

            // A sample to read Priorities data
            ReadPrioritiesData();

            Console.WriteLine();

            // A sample to create a Roundabout Site, set the Lane Disciplines specifically
            CreateRoundaboutSite();

            Console.WriteLine();

            // A sample to create a Roundabout Site, call the method SetDefaultLaneDisciplines() to automatically set the default Lane Disciplines on all Approach Lanes
            CreateRoundaboutSite2();

            Console.ReadLine();
        }

        /// <summary>
        /// A sample to create two Signals Sites and build a Freeway Diamond Interchange Network
        /// </summary>
        static void CreateFreewayDiamondInterchangeNetwork()
        {
            try
            {
                _api = new SIAPI();

                _api.CreateAndOpenProject(@"C:\Temp\APITest.sip8", "APITest");

                ISIAPIProject project = _api.Project;

                ISIAPISite site209W = CreateSite209W(project);

                ISIAPISite site209E = CreateSite209E(project);

                ISIAPINetwork network = CreateNetwork(project);

                Console.WriteLine("Processing Network...");

                if (network.Process())
                {
                    var time = network.OutputNetwork.GeneratedTime;
                    Console.WriteLine("Processed network at {0} by version {1}.", time, network.OutputNetwork.GeneratedByVersion);
                }
                else
                {
                    Console.WriteLine(network.ProcessingError);
                }

                AddMoreRoute(network);

                //CreateSignalsSite(project);

                //CreateRoundaboutSite(project);

                _api.CloseProject();

                Console.WriteLine("\nFinished successfully.");
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                if (ex.InnerException != null)
                    Console.WriteLine(ex.InnerException.Message);
            }
            finally
            {
                if (_api != null)
                    _api.Close();
            }

        }

        static ISIAPISite CreateSite209W(ISIAPIProject project)
        {
            ISIAPISite site = project.AddSite(0, 1);
            site.Name = "209W";
            site.Intersectionid = "209W";

            Console.WriteLine("Created Signalised Site 209W.");

            // Include User Class 7, use it as "LV to Freeway"
            ISIAPIMovementClass mcLF = site.MovementClasses[7];
            mcLF.Is_included = true;
            mcLF.Base_class = 1;
            mcLF.Name = "LV to Freeway";
            mcLF.Display_id = "LF";

            // Include User Class 8, use it as "HV to Freeway"
            ISIAPIMovementClass mcHF = site.MovementClasses[8];
            mcHF.Is_included = true;
            mcHF.Base_class = 2;
            mcHF.Name = "HV to Freeway";
            mcHF.Display_id = "HF";

            ISIAPILeg leg = null;
            int position = 1;
            ISIAPILaneApproach laneAppr = null;
            ISIAPILaneApproachMovement laneApprMov = null;
            ISIAPIIsland island = null;
            ISIAPILaneExit laneExit = null;
            ISIAPIMovement_vehicle_od movVehOD = null;
            ISIAPILaneMovement laneMov = null;

            // Add a Leg on SouthEast
            leg = site.AddLeg(1);
            leg.Name = "Freeway Off-Ramp South";
            leg.Approach_distance = 200;

            position = 1;

            // Add an Approach Lane
            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Type = 2; // Slip/Bypass (High Angle)
            laneAppr.Slip_control_type = 1; // Signals Control Type
            laneAppr.Width = 3.5f;
            laneAppr.Grade = 3;

            // Set the Lane Discipline from this Approach Lane to the West (destination: 6) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
            laneApprMov = laneAppr.LaneApproachMovements[6];
            laneApprMov.LaneApproachMovementMCs[1].Exists = true;
            laneApprMov.LaneApproachMovementMCs[2].Exists = true;

            // Add an Approach Lane and set the Discipline to West
            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Type = 2; // Slip/Bypass (High Angle)
            laneAppr.Slip_control_type = 1; // Signals Control Type
            laneAppr.Width = 3.5f;
            laneAppr.Grade = 3;

            laneApprMov = laneAppr.LaneApproachMovements[6];
            laneApprMov.LaneApproachMovementMCs[1].Exists = true;
            laneApprMov.LaneApproachMovementMCs[2].Exists = true;

            // Add an Approach Lane and set the Discipline to East
            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Width = 3.5f;
            laneAppr.Grade = 3;

            laneApprMov = laneAppr.LaneApproachMovements[2];
            laneApprMov.LaneApproachMovementMCs[1].Exists = true;
            laneApprMov.LaneApproachMovementMCs[2].Exists = true;

            // Add a Leg on East
            leg = site.AddLeg(2);
            leg.Name = "Westbound Internal";
            leg.Approach_distance = 70;

            position = 1;

            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Width = 3.5f;

            laneApprMov = laneAppr.LaneApproachMovements[6];
            laneApprMov.LaneApproachMovementMCs[1].Exists = true;
            laneApprMov.LaneApproachMovementMCs[2].Exists = true;

            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Width = 3.5f;

            laneApprMov = laneAppr.LaneApproachMovements[6];
            laneApprMov.LaneApproachMovementMCs[1].Exists = true;
            laneApprMov.LaneApproachMovementMCs[2].Exists = true;

            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Width = 3.5f;

            laneApprMov = laneAppr.LaneApproachMovements[6];
            laneApprMov.LaneApproachMovementMCs[1].Exists = true;
            laneApprMov.LaneApproachMovementMCs[2].Exists = true;

            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Width = 3.5f;

            laneApprMov = laneAppr.LaneApproachMovements[3];
            laneApprMov.LaneApproachMovementMCs[7].Exists = true; // set User Class 7 Discipline to North East Leg
            laneApprMov.LaneApproachMovementMCs[8].Exists = true; // set User Class 8 Discipline to North East Leg

            // Add an Island
            island = leg.InsertIsland(position++);
            island.Width = 1.5f;
            island.Width_back = 1.5f;

            // Add four Exit Lanes
            laneExit = leg.InsertExitLane(position++);
            laneExit.Width = 3.5f;

            laneExit = leg.InsertExitLane(position++);
            laneExit.Width = 3.5f;

            laneExit = leg.InsertExitLane(position++);
            laneExit.Width = 3.5f;

            laneExit = leg.InsertExitLane(position++);
            laneExit.Width = 3.5f;


            // Add a Leg on NorthEast
            leg = site.AddLeg(3);
            leg.Name = "Freeway On-Ramp North";
            leg.Approach_distance = 200;
            leg.Exit_distance_user = true;
            leg.Exit_distance = 200;

            position = 1;

            laneExit = leg.InsertExitLane(position++);
            laneExit.Width = 3.5f;
            laneExit.Grade = -3;


            // Add a Leg on West
            leg = site.AddLeg(6);
            leg.Name = "Eastbound External";
            leg.Approach_distance = 500;

            position = 1;

            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Configuration = 2; // Short Lane
            laneAppr.Type = 2; // Slip/Bypass (High Angle)
            laneAppr.Slip_control_type = 2; // Giveway/Yield Control Type
            laneAppr.Width = 3.5f;

            laneAppr.Length = 90;
            laneAppr.Segment1.Overflow_merge_lane_num_1 = 2;  // Overflow to Lane 2

            laneAppr.Is_sliplane_excluded_from_signal_analysis = true;

            laneApprMov = laneAppr.LaneApproachMovements[3];
            laneApprMov.LaneApproachMovementMCs[1].Exists = true;
            laneApprMov.LaneApproachMovementMCs[2].Exists = true;

            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Width = 3.5f;

            laneApprMov = laneAppr.LaneApproachMovements[2];
            laneApprMov.LaneApproachMovementMCs[1].Exists = true;
            laneApprMov.LaneApproachMovementMCs[2].Exists = true;

            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Width = 3.5f;

            laneApprMov = laneAppr.LaneApproachMovements[2];
            laneApprMov.LaneApproachMovementMCs[1].Exists = true;
            laneApprMov.LaneApproachMovementMCs[2].Exists = true;

            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Width = 3.5f;

            laneApprMov = laneAppr.LaneApproachMovements[2];
            laneApprMov.LaneApproachMovementMCs[1].Exists = true;
            laneApprMov.LaneApproachMovementMCs[2].Exists = true;

            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Configuration = 2; // Short Lane
            laneAppr.Width = 3.5f;
            laneAppr.Length = 90;
            laneAppr.Segment1.Overflow_merge_lane_num_1 = 4;  // Overflow to Lane 4

            laneApprMov = laneAppr.LaneApproachMovements[2];
            laneApprMov.LaneApproachMovementMCs[7].Exists = true;
            laneApprMov.LaneApproachMovementMCs[8].Exists = true;

            island = leg.InsertIsland(position++);
            island.Width = 5;
            island.Width_back = 8.5f;

            laneExit = leg.InsertExitLane(position++);
            laneExit.Width = 3.5f;
            laneExit = leg.InsertExitLane(position++);
            laneExit.Width = 3.5f;
            laneExit = leg.InsertExitLane(position++);
            laneExit.Width = 3.5f;

            Console.WriteLine("Updating Geometry Data...");

            // Call the method UpdateGeometryData() after Leg(s) and Lane(s) are added or removed.
            site.UpdateGeometryData();

            Console.WriteLine("Updated Geometry Data.");

            // Change the Lane Movement Flow Proportions from SouthEast Approach Lane 3 to East Exit Lane 1, 2, 3, 4
            // Lane Movement data can be accessed only after site.UpdateGeometryData() is called.
            laneAppr = site.Legs[1].LaneApproachs[3];
            laneMov = laneAppr.LaneMovements[2, 1];
            laneMov.LaneMovementMCs[1].Flow_proportion = 33;
            laneMov.LaneMovementMCs[2].Flow_proportion = 33;

            laneMov = laneAppr.LaneMovements[2, 2];
            laneMov.LaneMovementMCs[1].Flow_proportion = 34;
            laneMov.LaneMovementMCs[2].Flow_proportion = 34;

            laneMov = laneAppr.LaneMovements[2, 3];
            laneMov.LaneMovementMCs[1].Flow_proportion = 33;
            laneMov.LaneMovementMCs[2].Flow_proportion = 33;

            laneMov = laneAppr.LaneMovements[2, 4]; // by default, this Lane Moveemnt Flow Proportion is 100%, change it to 0.
            laneMov.LaneMovementMCs[1].Flow_proportion = 0;
            laneMov.LaneMovementMCs[2].Flow_proportion = 0;

            // Set Vehicle Origin-Destination (OD) Movements data, e.g. Volumes, Approach Cruise Speed and Exit Cruise Speed 
            movVehOD = site.MovementVehicleODs[1, 6];  // Vehicle OD Movement SouthEast to West
            movVehOD.Display_od_id = "1W";
            movVehOD.MovementVehicleODMCs[1].Volume = 830;  // Light Vehicle Movement Volume
            movVehOD.MovementVehicleODMCs[2].Volume = 25;   // Heavy Vehicle Movement Volume
            movVehOD.MovementVehicleODMCs[1].Peak_flow_factor = 100;
            movVehOD.MovementVehicleODMCs[2].Peak_flow_factor = 100;

            movVehOD.MovementVehicleODMCs[1].Approach_speed = 70;
            movVehOD.MovementVehicleODMCs[2].Approach_speed = 70;
            movVehOD.MovementVehicleODMCs[1].Exit_speed = 70;
            movVehOD.MovementVehicleODMCs[2].Exit_speed = 70;

            movVehOD = site.MovementVehicleODs[1, 3];   // Vehicle OD Movement SouthEast to NorthEast
            movVehOD.Exists = false; // set it not existing

            movVehOD = site.MovementVehicleODs[1, 2];   // Vehicle OD Movement SouthEast to East
            movVehOD.Display_od_id = "2W";
            movVehOD.MovementVehicleODMCs[1].Volume = 39;
            movVehOD.MovementVehicleODMCs[2].Volume = 3;
            movVehOD.MovementVehicleODMCs[1].Peak_flow_factor = 100;
            movVehOD.MovementVehicleODMCs[2].Peak_flow_factor = 100;

            movVehOD.MovementVehicleODMCs[1].Approach_speed = 70;
            movVehOD.MovementVehicleODMCs[2].Approach_speed = 70;
            movVehOD.MovementVehicleODMCs[1].Exit_speed = 40;
            movVehOD.MovementVehicleODMCs[2].Exit_speed = 40;

            movVehOD = site.MovementVehicleODs[2, 6];  // Vehicle OD Movement East to West
            movVehOD.Display_od_id = "3W";
            movVehOD.MovementVehicleODMCs[1].Volume = 1134;
            movVehOD.MovementVehicleODMCs[2].Volume = 36;
            movVehOD.MovementVehicleODMCs[1].Peak_flow_factor = 100;
            movVehOD.MovementVehicleODMCs[2].Peak_flow_factor = 100;

            movVehOD.MovementVehicleODMCs[1].Approach_speed = 40;
            movVehOD.MovementVehicleODMCs[2].Approach_speed = 40;
            movVehOD.MovementVehicleODMCs[1].Exit_speed = 70;
            movVehOD.MovementVehicleODMCs[2].Exit_speed = 70;

            movVehOD = site.MovementVehicleODs[2, 3];  // Vehicle OD Movement East to NorthEast
            movVehOD.Display_od_id = "4W";
            movVehOD.MovementVehicleODMCs[7].Volume = 215; // User Class 7 Movement Volume
            movVehOD.MovementVehicleODMCs[8].Volume = 7;   // User Class 8 Movement Volume
            movVehOD.MovementVehicleODMCs[7].Peak_flow_factor = 100;
            movVehOD.MovementVehicleODMCs[8].Peak_flow_factor = 100;

            movVehOD.MovementVehicleODMCs[7].Approach_speed = 40;
            movVehOD.MovementVehicleODMCs[8].Approach_speed = 40;
            movVehOD.MovementVehicleODMCs[7].Exit_speed = 70;
            movVehOD.MovementVehicleODMCs[8].Exit_speed = 70;

            movVehOD = site.MovementVehicleODs[6, 3];  // Vehicle OD Movement West to NorthEast
            movVehOD.Display_od_id = "5W";
            movVehOD.MovementVehicleODMCs[1].Volume = 280;
            movVehOD.MovementVehicleODMCs[2].Volume = 9;
            movVehOD.MovementVehicleODMCs[1].Peak_flow_factor = 100;
            movVehOD.MovementVehicleODMCs[2].Peak_flow_factor = 100;

            movVehOD.MovementVehicleODMCs[1].Approach_speed = 70;
            movVehOD.MovementVehicleODMCs[2].Approach_speed = 70;
            movVehOD.MovementVehicleODMCs[1].Exit_speed = 70;
            movVehOD.MovementVehicleODMCs[2].Exit_speed = 70;

            movVehOD = site.MovementVehicleODs[6, 2];  // Vehicle OD Movement West to East
            movVehOD.Display_od_id = "6W";
            movVehOD.MovementVehicleODMCs[1].Volume = 561;
            movVehOD.MovementVehicleODMCs[2].Volume = 15;
            movVehOD.MovementVehicleODMCs[7].Volume = 306;
            movVehOD.MovementVehicleODMCs[8].Volume = 10;
            movVehOD.MovementVehicleODMCs[1].Peak_flow_factor = 100;
            movVehOD.MovementVehicleODMCs[2].Peak_flow_factor = 100;
            movVehOD.MovementVehicleODMCs[7].Peak_flow_factor = 100;
            movVehOD.MovementVehicleODMCs[8].Peak_flow_factor = 100;

            movVehOD.MovementVehicleODMCs[1].Approach_speed = 70;
            movVehOD.MovementVehicleODMCs[2].Approach_speed = 70;
            movVehOD.MovementVehicleODMCs[7].Approach_speed = 70;
            movVehOD.MovementVehicleODMCs[8].Approach_speed = 70;
            movVehOD.MovementVehicleODMCs[1].Exit_speed = 40;
            movVehOD.MovementVehicleODMCs[2].Exit_speed = 40;
            movVehOD.MovementVehicleODMCs[7].Exit_speed = 40;
            movVehOD.MovementVehicleODMCs[8].Exit_speed = 40;

            // Turn off all Pedestrian Movements
            site.MovementPeds[1, 1, 0].Exists = false;
            site.MovementPeds[1, 2, 0].Exists = false;
            site.MovementPeds[1, 3, 0].Exists = false;
            site.MovementPeds[1, 6, 0].Exists = false;

            // Set the Phasing and Timing data
            // Add a new Sequence named "Diamond Interchange"
            ISIAPISequence sequence = site.AddSequence("Diamond Interchange");
            sequence.Is_selected = true;

            // Add a new Phase "A" to the Sequence
            ISIAPIPhase phase = sequence.AddPhase("A");
            phase.IsReferencePhase = true; // set it as the Referenced Phase

            phase.Phasemovement_vehicles[2, 6, 1].Running = true;
            phase.Phasemovement_vehicles[2, 6, 2].Running = true;
            phase.Phasemovement_vehicles[6, 3, 1].Running = true;
            phase.Phasemovement_vehicles[6, 3, 2].Running = true;
            phase.Phasemovement_vehicles[6, 2, 1].Running = true;
            phase.Phasemovement_vehicles[6, 2, 2].Running = true;
            phase.Phasemovement_vehicles[6, 2, 7].Running = true;
            phase.Phasemovement_vehicles[6, 2, 8].Running = true;

            phase = sequence.AddPhase("C");
            phase.Phasemovement_vehicles[2, 6, 1].Running = true;
            phase.Phasemovement_vehicles[2, 6, 2].Running = true;
            phase.Phasemovement_vehicles[2, 3, 7].Running = true;
            phase.Phasemovement_vehicles[2, 3, 8].Running = true;

            phase = sequence.AddPhase("D");
            phase.Phasemovement_vehicles[1, 6, 1].Running = true;
            phase.Phasemovement_vehicles[1, 6, 2].Running = true;
            phase.Phasemovement_vehicles[1, 2, 1].Running = true;
            phase.Phasemovement_vehicles[1, 2, 2].Running = true;

            _api.SubmitDataChanges();

            return site;
        }

        static ISIAPISite CreateSite209E(ISIAPIProject project)
        {
            ISIAPISite site = project.AddSite(0, 1);
            site.Name = "209E";
            site.Intersectionid = "209E";

            Console.WriteLine("Created Signalised Site 209E.");

            // Include User Class 7, use it as "LV to Freeway"
            ISIAPIMovementClass mcLF = site.MovementClasses[7];
            mcLF.Is_included = true;
            mcLF.Base_class = 1;
            mcLF.Name = "LV to Freeway";
            mcLF.Display_id = "LF";

            // Include User Class 8, use it as "HV to Freeway"
            ISIAPIMovementClass mcHF = site.MovementClasses[8];
            mcHF.Is_included = true;
            mcHF.Base_class = 2;
            mcHF.Name = "HV to Freeway";
            mcHF.Display_id = "HF";

            ISIAPILeg leg = null;
            int position = 1;
            ISIAPILaneApproach laneAppr = null;
            ISIAPILaneApproachMovement laneApprMov = null;
            ISIAPIIsland island = null;
            ISIAPILaneExit laneExit = null;
            ISIAPILaneMovement laneMov = null;
            ISIAPIMovement_vehicle_od movVehOD = null;

            // Add a Leg on East
            leg = site.AddLeg(2);
            leg.Name = "Westbound External";
            leg.Approach_distance = 500;

            position = 1;

            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Configuration = 2; // Short Lane
            laneAppr.Type = 2; // Slip/Bypass (High Angle)
            laneAppr.Slip_control_type = 2; // Giveway/Yield Control Type
            laneAppr.Width = 3.5f;

            laneAppr.Length = 90;
            laneAppr.Segment1.Overflow_merge_lane_num_1 = 2;

            laneAppr.Is_sliplane_excluded_from_signal_analysis = true;

            laneApprMov = laneAppr.LaneApproachMovements[7];
            laneApprMov.LaneApproachMovementMCs[1].Exists = true;
            laneApprMov.LaneApproachMovementMCs[2].Exists = true;

            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Width = 3.5f;

            laneApprMov = laneAppr.LaneApproachMovements[6];
            laneApprMov.LaneApproachMovementMCs[1].Exists = true;
            laneApprMov.LaneApproachMovementMCs[2].Exists = true;

            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Width = 3.5f;

            laneApprMov = laneAppr.LaneApproachMovements[6];
            laneApprMov.LaneApproachMovementMCs[1].Exists = true;
            laneApprMov.LaneApproachMovementMCs[2].Exists = true;

            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Width = 3.5f;

            laneApprMov = laneAppr.LaneApproachMovements[6];
            laneApprMov.LaneApproachMovementMCs[1].Exists = true;
            laneApprMov.LaneApproachMovementMCs[2].Exists = true;

            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Configuration = 2; // Short Lane
            laneAppr.Width = 3.5f;
            laneAppr.Length = 90;
            laneAppr.Segment1.Overflow_merge_lane_num_1 = 4;

            laneApprMov = laneAppr.LaneApproachMovements[6];
            laneApprMov.LaneApproachMovementMCs[7].Exists = true;
            laneApprMov.LaneApproachMovementMCs[8].Exists = true;

            // Add an Island
            island = leg.InsertIsland(position++);
            island.Width = 5;
            island.Width_back = 8.5f;

            // Add three Exit Lanes
            laneExit = leg.InsertExitLane(position++);
            laneExit.Width = 3.5f;
            laneExit = leg.InsertExitLane(position++);
            laneExit.Width = 3.5f;
            laneExit = leg.InsertExitLane(position++);
            laneExit.Width = 3.5f;

            // Add a Leg on NorthWest
            leg = site.AddLeg(5);
            leg.Name = "Freeway Off-Ramp North";
            leg.Approach_distance = 200;

            position = 1;

            // Add an Approach Lane and set the Discipline to East
            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Type = 2; // Slip/Bypass (High Angle)
            laneAppr.Slip_control_type = 2; // Giveway/Yield Control Type
            laneAppr.Width = 3.5f;
            laneAppr.Grade = 3;

            laneAppr.Is_sliplane_excluded_from_signal_analysis = true;

            laneApprMov = laneAppr.LaneApproachMovements[2];
            laneApprMov.LaneApproachMovementMCs[1].Exists = true;
            laneApprMov.LaneApproachMovementMCs[2].Exists = true;

            // Add an Approach Lane and set the Discipline to West
            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Width = 3.5f;
            laneAppr.Grade = 3;

            laneApprMov = laneAppr.LaneApproachMovements[6];
            laneApprMov.LaneApproachMovementMCs[1].Exists = true;
            laneApprMov.LaneApproachMovementMCs[2].Exists = true;


            // Add a Leg on West
            leg = site.AddLeg(6);
            leg.Name = "Eastbound Internal";
            leg.Approach_distance = 70;

            position = 1;

            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Width = 3.5f;

            laneApprMov = laneAppr.LaneApproachMovements[2];
            laneApprMov.LaneApproachMovementMCs[1].Exists = true;
            laneApprMov.LaneApproachMovementMCs[2].Exists = true;

            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Width = 3.5f;

            laneApprMov = laneAppr.LaneApproachMovements[2];
            laneApprMov.LaneApproachMovementMCs[1].Exists = true;
            laneApprMov.LaneApproachMovementMCs[2].Exists = true;

            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Width = 3.5f;

            laneApprMov = laneAppr.LaneApproachMovements[2];
            laneApprMov.LaneApproachMovementMCs[1].Exists = true;
            laneApprMov.LaneApproachMovementMCs[2].Exists = true;

            laneAppr = leg.InsertApproachLane(position++);
            laneAppr.Width = 3.5f;

            laneApprMov = laneAppr.LaneApproachMovements[7];
            laneApprMov.LaneApproachMovementMCs[7].Exists = true; // set User Class 7 Discipline to SouthWest Leg
            laneApprMov.LaneApproachMovementMCs[8].Exists = true; // set User Class 8 Discipline to SouthWest Leg

            // Add an Island
            island = leg.InsertIsland(position++);
            island.Width = 1.5f;
            island.Width_back = 1.5f;

            // Add four Exit Lanes
            laneExit = leg.InsertExitLane(position++);
            laneExit.Width = 3.5f;

            laneExit = leg.InsertExitLane(position++);
            laneExit.Width = 3.5f;

            laneExit = leg.InsertExitLane(position++);
            laneExit.Width = 3.5f;

            laneExit = leg.InsertExitLane(position++);
            laneExit.Width = 3.5f;

            // Add a Leg on SouthWest
            leg = site.AddLeg(7);
            leg.Name = "Freeway On-Ramp South";
            leg.Approach_distance = 200;
            leg.Exit_distance_user = true;
            leg.Exit_distance = 200;

            position = 1;

            laneExit = leg.InsertExitLane(position++);
            laneExit.Width = 3.5f;
            laneExit.Grade = -3;


            Console.WriteLine("Updating Geometry Data...");

            // Call the method UpdateGeometryData() after Leg(s) and Lane(s) are added or removed.
            site.UpdateGeometryData();

            Console.WriteLine("Updated Geometry Data.");

            // Set the Lane Movement Flow Proportions from NorthWest Approach Lane 2 to West Exit Lane 1, 2, 3
            // Lane Movement data can be accessed only after site.UpdateGeometryData() is called.
            laneAppr = site.Legs[5].LaneApproachs[2];
            laneMov = laneAppr.LaneMovements[6, 1];
            laneMov.LaneMovementMCs[1].Flow_proportion = 33;
            laneMov.LaneMovementMCs[2].Flow_proportion = 33;

            laneMov = laneAppr.LaneMovements[6, 2];
            laneMov.LaneMovementMCs[1].Flow_proportion = 34;
            laneMov.LaneMovementMCs[2].Flow_proportion = 34;

            laneMov = laneAppr.LaneMovements[6, 3];
            laneMov.LaneMovementMCs[1].Flow_proportion = 33;
            laneMov.LaneMovementMCs[2].Flow_proportion = 33;

            laneMov = laneAppr.LaneMovements[6, 4]; // by default, this Lane Moveemnt Flow Proportion is 100%, change it to 0.
            laneMov.LaneMovementMCs[1].Flow_proportion = 0;
            laneMov.LaneMovementMCs[2].Flow_proportion = 0;

            // Set Vehicle Origin-Destination (OD) Movements data, e.g. Volumes, Approach Cruise Speed and Exit Cruise Speed 
            movVehOD = site.MovementVehicleODs[5, 2];  // Vehicle OD Movement NorthWest to East
            movVehOD.Display_od_id = "1E";
            movVehOD.MovementVehicleODMCs[1].Volume = 186;  // Light Vehicle Movement Volume
            movVehOD.MovementVehicleODMCs[2].Volume = 8;    // Heavy Vehicle Movement Volume
            movVehOD.MovementVehicleODMCs[1].Peak_flow_factor = 100;
            movVehOD.MovementVehicleODMCs[2].Peak_flow_factor = 100;

            movVehOD.MovementVehicleODMCs[1].Approach_speed = 70;
            movVehOD.MovementVehicleODMCs[2].Approach_speed = 70;
            movVehOD.MovementVehicleODMCs[1].Exit_speed = 70;
            movVehOD.MovementVehicleODMCs[2].Exit_speed = 70;

            movVehOD = site.MovementVehicleODs[5, 7];   // Vehicle OD Movement NorthWest to SouthWest
            movVehOD.Exists = false; // set it not existing

            movVehOD = site.MovementVehicleODs[5, 6];   // Vehicle OD Movement NorthWest to West
            movVehOD.Display_od_id = "2E";
            movVehOD.MovementVehicleODMCs[1].Volume = 192;
            movVehOD.MovementVehicleODMCs[2].Volume = 6;
            movVehOD.MovementVehicleODMCs[1].Peak_flow_factor = 100;
            movVehOD.MovementVehicleODMCs[2].Peak_flow_factor = 100;

            movVehOD.MovementVehicleODMCs[1].Approach_speed = 70;
            movVehOD.MovementVehicleODMCs[2].Approach_speed = 70;
            movVehOD.MovementVehicleODMCs[1].Exit_speed = 40;
            movVehOD.MovementVehicleODMCs[2].Exit_speed = 40;

            movVehOD = site.MovementVehicleODs[6, 2];  // Vehicle OD Movement West to East
            movVehOD.Display_od_id = "3E";
            movVehOD.MovementVehicleODMCs[1].Volume = 600;
            movVehOD.MovementVehicleODMCs[2].Volume = 18;
            movVehOD.MovementVehicleODMCs[1].Peak_flow_factor = 100;
            movVehOD.MovementVehicleODMCs[2].Peak_flow_factor = 100;

            movVehOD.MovementVehicleODMCs[1].Approach_speed = 40;
            movVehOD.MovementVehicleODMCs[2].Approach_speed = 40;
            movVehOD.MovementVehicleODMCs[1].Exit_speed = 70;
            movVehOD.MovementVehicleODMCs[2].Exit_speed = 70;

            movVehOD = site.MovementVehicleODs[6, 7];  // Vehicle OD Movement West to SouthWest
            movVehOD.Display_od_id = "4E";
            movVehOD.MovementVehicleODMCs[7].Volume = 306; // User Class 7 Movement Volume
            movVehOD.MovementVehicleODMCs[8].Volume = 10;  // User Class 8 Movement Volume
            movVehOD.MovementVehicleODMCs[7].Peak_flow_factor = 100;
            movVehOD.MovementVehicleODMCs[8].Peak_flow_factor = 100;

            movVehOD.MovementVehicleODMCs[7].Approach_speed = 40;
            movVehOD.MovementVehicleODMCs[8].Approach_speed = 40;
            movVehOD.MovementVehicleODMCs[7].Exit_speed = 70;
            movVehOD.MovementVehicleODMCs[8].Exit_speed = 70;

            movVehOD = site.MovementVehicleODs[2, 7];  // Vehicle OD Movement East to SouthWest
            movVehOD.Display_od_id = "5E";
            movVehOD.MovementVehicleODMCs[1].Volume = 120;
            movVehOD.MovementVehicleODMCs[2].Volume = 4;
            movVehOD.MovementVehicleODMCs[1].Peak_flow_factor = 100;
            movVehOD.MovementVehicleODMCs[2].Peak_flow_factor = 100;

            movVehOD.MovementVehicleODMCs[1].Approach_speed = 70;
            movVehOD.MovementVehicleODMCs[2].Approach_speed = 70;
            movVehOD.MovementVehicleODMCs[1].Exit_speed = 70;
            movVehOD.MovementVehicleODMCs[2].Exit_speed = 70;

            movVehOD = site.MovementVehicleODs[2, 6];  // Vehicle OD Movement East to West
            movVehOD.Display_od_id = "6E";
            movVehOD.MovementVehicleODMCs[1].Volume = 942;
            movVehOD.MovementVehicleODMCs[2].Volume = 30;
            movVehOD.MovementVehicleODMCs[7].Volume = 215;
            movVehOD.MovementVehicleODMCs[8].Volume = 7;
            movVehOD.MovementVehicleODMCs[1].Peak_flow_factor = 100;
            movVehOD.MovementVehicleODMCs[2].Peak_flow_factor = 100;
            movVehOD.MovementVehicleODMCs[7].Peak_flow_factor = 100;
            movVehOD.MovementVehicleODMCs[8].Peak_flow_factor = 100;

            movVehOD.MovementVehicleODMCs[1].Approach_speed = 70;
            movVehOD.MovementVehicleODMCs[2].Approach_speed = 70;
            movVehOD.MovementVehicleODMCs[7].Approach_speed = 70;
            movVehOD.MovementVehicleODMCs[8].Approach_speed = 70;
            movVehOD.MovementVehicleODMCs[1].Exit_speed = 40;
            movVehOD.MovementVehicleODMCs[2].Exit_speed = 40;
            movVehOD.MovementVehicleODMCs[7].Exit_speed = 40;
            movVehOD.MovementVehicleODMCs[8].Exit_speed = 40;

            // Turn off all Pedestrian Movements
            site.MovementPeds[1, 2, 0].Exists = false;
            site.MovementPeds[1, 5, 0].Exists = false;
            site.MovementPeds[1, 6, 0].Exists = false;
            site.MovementPeds[1, 7, 0].Exists = false;

            // Set the Phasing and Timing data
            // Add a new Sequence named "Diamond Interchange"
            ISIAPISequence sequence = site.AddSequence("Diamond Interchange");
            sequence.Is_selected = true;

            // Add a new Phase "A" to the Sequence
            ISIAPIPhase phase = sequence.AddPhase("A");
            phase.IsReferencePhase = true; // set it as the Referenced Phase

            phase.Phasemovement_vehicles[2, 6, 1].Running = true;
            phase.Phasemovement_vehicles[2, 6, 2].Running = true;
            phase.Phasemovement_vehicles[2, 6, 7].Running = true;
            phase.Phasemovement_vehicles[2, 6, 8].Running = true;
            phase.Phasemovement_vehicles[6, 2, 1].Running = true;
            phase.Phasemovement_vehicles[6, 2, 2].Running = true;

            phase = sequence.AddPhase("C");
            phase.Phasemovement_vehicles[6, 2, 1].Running = true;
            phase.Phasemovement_vehicles[6, 2, 2].Running = true;
            phase.Phasemovement_vehicles[6, 7, 7].Running = true;
            phase.Phasemovement_vehicles[6, 7, 8].Running = true;

            phase = sequence.AddPhase("D");
            phase.Phasemovement_vehicles[5, 6, 1].Running = true;
            phase.Phasemovement_vehicles[5, 6, 2].Running = true;

            _api.SubmitDataChanges();

            return site;
        }

        static ISIAPINetwork CreateNetwork(ISIAPIProject project)
        {
            ISIAPINetwork network = project.AddNetwork(1);
            network.Name = "Freeway Diamond Interchange";
            network.NetworkID = "209";

            Console.WriteLine("Created Network \"Freeway Diamond Interchange\".");

            network.DesiredSpeedMethod = 0; // User-input
            network.DesiredSpeed = 70;

            ISIAPINetworkSite nwSite209W = network.AddNetworkSite(project.Sites["209W"], 5200, 5100);
            ISIAPINetworkSite nwSite209E = network.AddNetworkSite(project.Sites["209E"], 5470, 5100);

            network.AddNetworkLegConnection(nwSite209W, 2, nwSite209E, 6);

            ISIAPINetworkCCG ccg = network.AddNetworkCCG();
            ccg.Name = "Fwy Int";
            ccg.NetworkCCGID = "209";

            ccg.AddNetworkSite(nwSite209W);
            ccg.AddNetworkSite(nwSite209E);

            ISIAPINetworkCCGSequence ccgSeq = ccg.AddCCGSequence("CCG Phasing");
            ccgSeq.Is_selected = true;
            ccgSeq.Cycle_time_option = 2; // User-Given Cycle Time
            ccgSeq.Usergiven_cycle_time = 90;
            ccgSeq.Green_split_priority_option = 1;

            ISIAPINetworkCCGPhase ccgPhase = ccgSeq.AddCCGPhase("A");
            ccgPhase.IsReferencePhase = true;

            ISIAPIPhasemovement_vehicles phaseMovVehs = ccgPhase.GetPhasemovementVehiclesByNetworkSite(nwSite209W);
            phaseMovVehs[2, 6, 1].Running = true;
            phaseMovVehs[2, 6, 2].Running = true;
            phaseMovVehs[6, 2, 1].Running = true;
            phaseMovVehs[6, 2, 2].Running = true;
            phaseMovVehs[6, 2, 7].Running = true;
            phaseMovVehs[6, 2, 8].Running = true;

            phaseMovVehs = ccgPhase.GetPhasemovementVehiclesByNetworkSite(nwSite209E);
            phaseMovVehs[2, 6, 1].Running = true;
            phaseMovVehs[2, 6, 2].Running = true;
            phaseMovVehs[2, 6, 7].Running = true;
            phaseMovVehs[2, 6, 8].Running = true;
            phaseMovVehs[6, 2, 1].Running = true;
            phaseMovVehs[6, 2, 2].Running = true;

            ccgPhase = ccgSeq.AddCCGPhase("C");

            phaseMovVehs = ccgPhase.GetPhasemovementVehiclesByNetworkSite(nwSite209W);
            phaseMovVehs[2, 6, 1].Running = true;
            phaseMovVehs[2, 6, 2].Running = true;
            phaseMovVehs[2, 3, 7].Running = true;
            phaseMovVehs[2, 3, 8].Running = true;

            phaseMovVehs = ccgPhase.GetPhasemovementVehiclesByNetworkSite(nwSite209E);
            phaseMovVehs[6, 2, 1].Running = true;
            phaseMovVehs[6, 2, 2].Running = true;
            phaseMovVehs[6, 7, 7].Running = true;
            phaseMovVehs[6, 7, 8].Running = true;

            ccgPhase = ccgSeq.AddCCGPhase("D");

            phaseMovVehs = ccgPhase.GetPhasemovementVehiclesByNetworkSite(nwSite209W);
            phaseMovVehs[1, 6, 1].Running = true;
            phaseMovVehs[1, 6, 2].Running = true;
            phaseMovVehs[1, 2, 1].Running = true;
            phaseMovVehs[1, 2, 2].Running = true;

            phaseMovVehs = ccgPhase.GetPhasemovementVehiclesByNetworkSite(nwSite209E);
            phaseMovVehs[5, 6, 1].Running = true;
            phaseMovVehs[5, 6, 2].Running = true;

            Console.WriteLine("Added CCG and its Phasing data");

            ISIAPIRoute route = network.AddRoute();
            route.AddRouteNwSite(nwSite209W, 6, 2);
            route.AddRouteNwSite(nwSite209E, 6, 2);

            route.CheckIsValidForSignalOffsetCalc();

            route.IsIncludedInOutputNetworkByRoutes = true;

            _api.SubmitDataChanges();

            return network;
        }

        static void AddMoreRoute(ISIAPINetwork network)
        {
            ISIAPINetworkSite nwSite209W = network.NetworkSites["209W"];
            ISIAPINetworkSite nwSite209E = network.NetworkSites["209E"];

            ISIAPIRoute route = network.AddRoute();
            route.AddRouteNwSite(nwSite209E, 2, 6);
            route.AddRouteNwSite(nwSite209W, 2, 6);

            route.CheckIsValidForSignalOffsetCalc();

            route.IsIncludedInOutputNetworkByRoutes = true;

            _api.SubmitDataChanges();

            if (network.OutputNetwork != null)
            {
                route.Process();

                network.ProcessByRoutes();
            }
        }

        static void CreateSignalsSite()
        {
            try
            {
                _api = new SIAPI();

                _api.CreateAndOpenProject(@"C:\Temp\Signals.sip8", "Signals");

                ISIAPIProject project = _api.Project;

                ISIAPISite site = project.AddSite(0, 4);
                site.Name = "Signals";

                Console.WriteLine("Created Signalised Site.");

                // Add a Leg on South
                ISIAPILeg leg = site.AddLeg(0);
                leg.Name = "Smith Road";
                int position = 1;

                // Add two Exit Lanes
                leg.InsertExitLane(position++);
                leg.InsertExitLane(position++);

                // Add an Island
                ISIAPIIsland island = leg.InsertIsland(position++);
                island.Width_back = 5.3f;

                // Add an Approach Lane
                ISIAPILaneApproach laneAppr = leg.InsertApproachLane(position++);

                // Set the Lane Discipline from this Approach Lane to the West (destination: 6) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
                ISIAPILaneApproachMovement laneApprMov = laneAppr.LaneApproachMovements[6];
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;

                // Set the Approach Lane as a short Lane
                laneAppr.Configuration = 2;
                laneAppr.Length = 100;

                // Add an Approach Lane and set the Discipline to North
                laneAppr = leg.InsertApproachLane(position++);
                laneApprMov = laneAppr.LaneApproachMovements[4];
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;

                // Add an Approach Lane and set the Discipline to North
                laneAppr = leg.InsertApproachLane(position++);
                laneApprMov = laneAppr.LaneApproachMovements[4];
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;

                // Add a Leg on North and then add two Exit Lanes and two Approach Lanes
                leg = site.AddLeg(4);
                leg.Name = "Smith Road";
                position = 1;

                leg.InsertExitLane(position++);
                leg.InsertExitLane(position++);

                leg.InsertIsland(position++);

                laneAppr = leg.InsertApproachLane(position++);
                laneApprMov = laneAppr.LaneApproachMovements[0];
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;

                laneAppr = leg.InsertApproachLane(position++);

                // Set the Lane Disciplines to South and West
                laneApprMov = laneAppr.LaneApproachMovements[0];
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;
                laneApprMov = laneAppr.LaneApproachMovements[6];
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;

                // Add a Leg on West and then add two Exit Lanes and two Approach Lanes
                leg = site.AddLeg(6);
                leg.Name = "Park Road";
                position = 1;

                leg.InsertExitLane(position++);
                leg.InsertExitLane(position++);

                leg.InsertIsland(position++);

                laneAppr = leg.InsertApproachLane(position++);
                laneApprMov = laneAppr.LaneApproachMovements[4];
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;

                laneAppr = leg.InsertApproachLane(position++);
                laneApprMov = laneAppr.LaneApproachMovements[0];
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;

                Console.WriteLine("Updating Geometry Data...");

                // Call the method UpdateGeometryData() after Leg(s) and Lane(s) are added or removed.
                site.UpdateGeometryData();

                Console.WriteLine("Updated Geometry Data.");

                // Set the Light Vehicle and Heavy Vehicle Volumes on the Origin-Destination (OD) Movements
                ISIAPIMovement_vehicle_od movVehOD = site.MovementVehicleODs[0, 6];
                movVehOD.MovementVehicleODMCs[1].Volume = 25;
                movVehOD.MovementVehicleODMCs[2].Volume = 1;

                movVehOD = site.MovementVehicleODs[0, 4];
                movVehOD.MovementVehicleODMCs[1].Volume = 150;
                movVehOD.MovementVehicleODMCs[2].Volume = 3;

                movVehOD = site.MovementVehicleODs[4, 0];
                movVehOD.MovementVehicleODMCs[1].Volume = 150;
                movVehOD.MovementVehicleODMCs[2].Volume = 3;

                movVehOD = site.MovementVehicleODs[4, 6];
                movVehOD.MovementVehicleODMCs[1].Volume = 50;
                movVehOD.MovementVehicleODMCs[2].Volume = 1;

                // Set this OD Movement (North to West) is opposed by the West Pedestrian Full-crossing
                movVehOD.Opposingmovement_peds[1, 6, 0].Opposing = true;

                movVehOD = site.MovementVehicleODs[6, 4];
                movVehOD.MovementVehicleODMCs[1].Volume = 25;
                movVehOD.MovementVehicleODMCs[2].Volume = 1;

                movVehOD = site.MovementVehicleODs[6, 0];
                movVehOD.MovementVehicleODMCs[1].Volume = 100;
                movVehOD.MovementVehicleODMCs[2].Volume = 2;

                // Set this OD Movement (West to South) is opposed by the South Pedestrian Full-crossing
                movVehOD.Opposingmovement_peds[1, 0, 0].Opposing = true;

                // Enable the Full-crossing (type: 1) Pedestrian Movements on the Legs (movPed.Exists = true) and set the Volumes
                ISIAPIMovement_ped movPed = site.MovementPeds[1, 0, 0];
                movPed.Exists = true;
                movPed.Volume = 25;

                movPed = site.MovementPeds[1, 4, 0];
                movPed.Exists = true;
                movPed.Volume = 25;

                movPed = site.MovementPeds[1, 6, 0];
                movPed.Exists = true;
                movPed.Volume = 25;

                // Set the Phasing and Timing data
                // Add a new Sequence named "Split Phasing"
                ISIAPISequence sequence = site.AddSequence("Split Phasing");
                sequence.Is_selected = true;

                // Add a new Phase "A" to the Sequence
                ISIAPIPhase phase = sequence.AddPhase("A");
                phase.IsReferencePhase = true; // set it as the Referenced Phase

                // Set the South-to-West Movement and West-to-South Movement of Light Vehicle and Heavy Vehicle running on this Phase
                phase.Phasemovement_vehicles[0, 6, 1].Running = true;
                phase.Phasemovement_vehicles[0, 6, 2].Running = true;
                phase.Phasemovement_vehicles[6, 0, 1].Running = true;
                phase.Phasemovement_vehicles[6, 0, 2].Running = true;

                // Set the North Pedestrian Full-crossing Movement running on this Phase
                phase.Phasemovement_peds[1, 4, 0].Running = true;

                // Add a new Phase "B" to the Sequence and set the movements running on it
                phase = sequence.AddPhase("B");
                phase.Phasemovement_vehicles[0, 4, 1].Running = true;
                phase.Phasemovement_vehicles[0, 4, 2].Running = true;
                phase.Phasemovement_vehicles[4, 0, 1].Running = true;
                phase.Phasemovement_vehicles[4, 0, 2].Running = true;
                phase.Phasemovement_vehicles[4, 6, 1].Running = true;
                phase.Phasemovement_vehicles[4, 6, 2].Running = true;

                phase.Phasemovement_vehicles[4, 6, 1].Terminates = true;
                phase.Phasemovement_vehicles[4, 6, 2].Terminates = true;

                phase.Phasemovement_peds[1, 6, 0].Running = true;

                // Add a new Phase "C" to the Sequence and set the movements running on it
                phase = sequence.AddPhase("C");
                phase.Phasemovement_vehicles[4, 6, 1].Running = true;
                phase.Phasemovement_vehicles[4, 6, 2].Running = true;
                phase.Phasemovement_vehicles[6, 4, 1].Running = true;
                phase.Phasemovement_vehicles[6, 4, 2].Running = true;
                phase.Phasemovement_vehicles[6, 0, 1].Running = true;
                phase.Phasemovement_vehicles[6, 0, 2].Running = true;

                phase.Phasemovement_vehicles[6, 0, 1].Terminates = true;
                phase.Phasemovement_vehicles[6, 0, 2].Terminates = true;

                phase.Phasemovement_peds[1, 0, 0].Running = true;

                Console.WriteLine("Processing site...");
                if (site.Process())
                {
                    var time = site.Outputset.Generated;
                    Console.WriteLine("Processed site at {0}.", time);
                }
                else
                {
                    Console.WriteLine(site.ProcessingError);
                }

                Console.WriteLine("\nFinished successfully.");
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                if (ex.InnerException != null)
                    Console.WriteLine(ex.InnerException.Message);
            }
            finally
            {
                if (_api != null)
                    _api.Close();
            }
        }

        /// <summary>
        /// A sample to create a Roundabout Site, set the Lane Disciplines specifically
        /// </summary>
        static void CreateRoundaboutSite()
        {
            try
            {
                _api = new SIAPI();

                _api.CreateAndOpenProject(@"C:\Temp\Roundabout.sip8", "Roundabout");

                ISIAPIProject project = _api.Project;

                ISIAPISite site = project.AddSite(1, 4);
                site.Name = "Roundabout";

                Console.WriteLine("Created Roundabout Site '{0}'.", site.Name);

                // Add a Leg on South
                ISIAPILeg leg = site.AddLeg(0);
                leg.Name = "Eley Road";
                int legItemPosition = 1;

                // Add two Exit Lanes
                leg.InsertExitLane(legItemPosition++);
                leg.InsertExitLane(legItemPosition++);

                // A Roundabout Splitter Island must be added between Approach Lanes and Exit Lanes on each Two-way Leg on a Roundabout Site.
                // A normal Strip Island can be added (by calling the method leg.InsertIsland()) between two Approach Lanes or between two Exit Lanes if required for the geometry layout.
                leg.InsertIsland_RoundaboutSplitter(legItemPosition++);

                // Add an Approach Lane
                ISIAPILaneApproach laneAppr = leg.InsertApproachLane(legItemPosition++);

                // Set the Lane Discipline from this Approach Lane to the West (destination: 6) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
                ISIAPILaneApproachMovement laneApprMov = laneAppr.LaneApproachMovements[6];
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;

                // Set the Lane Discipline from this Approach Lane to the North (destination: 4) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
                laneApprMov = laneAppr.LaneApproachMovements[4];
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;

                // Add an Approach Lane
                laneAppr = leg.InsertApproachLane(legItemPosition++);

                // Set the Lane Discipline from this Approach Lane to the North (destination: 4) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
                laneApprMov = laneAppr.LaneApproachMovements[4];
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;

                // Set the Lane Discipline from this Approach Lane to the East (destination: 2) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
                laneApprMov = laneAppr.LaneApproachMovements[2];
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;

                // Add a Leg on East
                leg = site.AddLeg(2);
                leg.Name = "Park Road";
                legItemPosition = 1;

                // Add two Exit Lanes
                leg.InsertExitLane(legItemPosition++);
                leg.InsertExitLane(legItemPosition++);

                // Add a Roundabout Splitter Island
                leg.InsertIsland_RoundaboutSplitter(legItemPosition++);

                // Add two Approach Lanes and set the Lane Disciplines
                laneAppr = leg.InsertApproachLane(legItemPosition++);
                laneApprMov = laneAppr.LaneApproachMovements[0]; // Lane Disciplines to South
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;
                laneApprMov = laneAppr.LaneApproachMovements[6]; // Lane Disciplines to West
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;

                laneAppr = leg.InsertApproachLane(legItemPosition++);
                laneApprMov = laneAppr.LaneApproachMovements[6];  // Lane Disciplines to West
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;
                laneApprMov = laneAppr.LaneApproachMovements[4];  // Lane Disciplines to North
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;

                // Add a Leg on North
                leg = site.AddLeg(4);
                leg.Name = "Eley Road";
                legItemPosition = 1;

                leg.InsertExitLane(legItemPosition++);
                leg.InsertExitLane(legItemPosition++);

                leg.InsertIsland_RoundaboutSplitter(legItemPosition++);

                laneAppr = leg.InsertApproachLane(legItemPosition++);
                laneApprMov = laneAppr.LaneApproachMovements[2];
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;
                laneApprMov = laneAppr.LaneApproachMovements[0];
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;

                laneAppr = leg.InsertApproachLane(legItemPosition++);
                laneApprMov = laneAppr.LaneApproachMovements[0];
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;
                laneApprMov = laneAppr.LaneApproachMovements[6];
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;

                // Add a Leg on West
                leg = site.AddLeg(6);
                leg.Name = "Park Road";
                legItemPosition = 1;

                leg.InsertExitLane(legItemPosition++);
                leg.InsertExitLane(legItemPosition++);

                leg.InsertIsland_RoundaboutSplitter(legItemPosition++);

                laneAppr = leg.InsertApproachLane(legItemPosition++);
                laneApprMov = laneAppr.LaneApproachMovements[4];
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;
                laneApprMov = laneAppr.LaneApproachMovements[2];
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;

                laneAppr = leg.InsertApproachLane(legItemPosition++);
                laneApprMov = laneAppr.LaneApproachMovements[2];
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;
                laneApprMov = laneAppr.LaneApproachMovements[0];
                laneApprMov.LaneApproachMovementMCs[1].Exists = true;
                laneApprMov.LaneApproachMovementMCs[2].Exists = true;

                Console.WriteLine("Updating Geometry Data...");

                // Call the method UpdateGeometryData() after Leg(s) and Lane(s) are added or removed.
                site.UpdateGeometryData();

                Console.WriteLine("Updated Geometry Data.");

                // Set Vehicle Origin-Destination (OD) Movements data, e.g. Volumes
                ISIAPIMovement_vehicle_od movVehOD = site.MovementVehicleODs[0, 6];  // Vehicle OD Movement South to West
                movVehOD.MovementVehicleODMCs[1].Volume = 25;
                movVehOD.MovementVehicleODMCs[2].Volume = 1;

                movVehOD = site.MovementVehicleODs[0, 4];
                movVehOD.MovementVehicleODMCs[1].Volume = 150;
                movVehOD.MovementVehicleODMCs[2].Volume = 3;

                movVehOD = site.MovementVehicleODs[0, 2];
                movVehOD.MovementVehicleODMCs[1].Volume = 100;
                movVehOD.MovementVehicleODMCs[2].Volume = 2;

                movVehOD = site.MovementVehicleODs[2, 0];
                movVehOD.MovementVehicleODMCs[1].Volume = 25;
                movVehOD.MovementVehicleODMCs[2].Volume = 1;

                movVehOD = site.MovementVehicleODs[2, 6];
                movVehOD.MovementVehicleODMCs[1].Volume = 150;
                movVehOD.MovementVehicleODMCs[2].Volume = 3;

                movVehOD = site.MovementVehicleODs[2, 4];
                movVehOD.MovementVehicleODMCs[1].Volume = 100;
                movVehOD.MovementVehicleODMCs[2].Volume = 2;

                movVehOD = site.MovementVehicleODs[4, 2];
                movVehOD.MovementVehicleODMCs[1].Volume = 25;
                movVehOD.MovementVehicleODMCs[2].Volume = 1;

                movVehOD = site.MovementVehicleODs[4, 0];
                movVehOD.MovementVehicleODMCs[1].Volume = 150;
                movVehOD.MovementVehicleODMCs[2].Volume = 3;

                movVehOD = site.MovementVehicleODs[4, 6];
                movVehOD.MovementVehicleODMCs[1].Volume = 100;
                movVehOD.MovementVehicleODMCs[2].Volume = 2;

                movVehOD = site.MovementVehicleODs[6, 4];
                movVehOD.MovementVehicleODMCs[1].Volume = 25;
                movVehOD.MovementVehicleODMCs[2].Volume = 1;

                movVehOD = site.MovementVehicleODs[6, 2];
                movVehOD.MovementVehicleODMCs[1].Volume = 150;
                movVehOD.MovementVehicleODMCs[2].Volume = 3;

                movVehOD = site.MovementVehicleODs[6, 0];
                movVehOD.MovementVehicleODMCs[1].Volume = 100;
                movVehOD.MovementVehicleODMCs[2].Volume = 2;

                // Set Pedestrian Movement Data

                // Staged Crossing (Type: 3), South Leg (Origin: 0), Stage 1 (Approach Side)
                ISIAPIMovement_ped movPed = site.MovementPeds[3, 0, 1];
                movPed.Exists = true;
                movPed.Volume = 25;

                // Staged Crossing (Type: 3), South Leg (Origin: 0), Stage 2 (Exit Side)
                movPed = site.MovementPeds[3, 0, 2];
                movPed.Exists = true;
                movPed.Volume = 25;

                // Staged Crossing (Type: 3), East Leg (Origin: 2), Stage 1 (Approach Side)
                movPed = site.MovementPeds[3, 2, 1];
                movPed.Exists = true;
                movPed.Volume = 25;

                // Staged Crossing (Type: 3), East Leg (Origin: 2), Stage 2 (Exit Side)
                movPed = site.MovementPeds[3, 2, 2];
                movPed.Exists = true;
                movPed.Volume = 25;

                movPed = site.MovementPeds[3, 4, 1];
                movPed.Exists = true;
                movPed.Volume = 25;
                movPed = site.MovementPeds[3, 4, 2];
                movPed.Exists = true;
                movPed.Volume = 25;

                movPed = site.MovementPeds[3, 6, 1];
                movPed.Exists = true;
                movPed.Volume = 25;
                movPed = site.MovementPeds[3, 6, 2];
                movPed.Exists = true;
                movPed.Volume = 25;

                Console.WriteLine("Processing site...");
                if (site.Process())
                {
                    var time = site.Outputset.Generated;
                    Console.WriteLine("Processed site at {0}.", time);
                }
                else
                {
                    Console.WriteLine(site.ProcessingError);
                }

                Console.WriteLine("\nFinished successfully.");
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                if (ex.InnerException != null)
                    Console.WriteLine(ex.InnerException.Message);
            }
            finally
            {
                if (_api != null)
                    _api.Close();
            }
        }

        /// <summary>
        /// A sample to create a Roundabout Site, call the method SetDefaultLaneDisciplines() to automatically set the default Lane Disciplines on all Approach Lanes
        /// </summary>
        static void CreateRoundaboutSite2()
        {
            try
            {
                _api = new SIAPI();

                _api.CreateAndOpenProject(@"C:\Temp\Roundabout2.sip8", "Roundabout2");

                ISIAPIProject project = _api.Project;

                ISIAPISite site = project.AddSite(1, 4);
                site.Name = "Roundabout2";

                Console.WriteLine("Created Roundabout Site '{0}'.", site.Name);

                // Add a Leg on South
                ISIAPILeg leg = site.AddLeg(0);
                leg.Name = "Eley Road";
                int legItemPosition = 1;

                // Add three Exit Lanes
                leg.InsertExitLane(legItemPosition++);
                leg.InsertExitLane(legItemPosition++);
                leg.InsertExitLane(legItemPosition++);

                // A Roundabout Splitter Island must be added between Approach Lanes and Exit Lanes on each Two-way Leg on a Roundabout Site.
                // A normal Strip Island can be added (by calling the method leg.InsertIsland()) between two Approach Lanes or between two Exit Lanes if required for the geometry layout.
                leg.InsertIsland_RoundaboutSplitter(legItemPosition++);

                // Add three Approach Lanes
                ISIAPILaneApproach laneAppr = leg.InsertApproachLane(legItemPosition++);
                laneAppr = leg.InsertApproachLane(legItemPosition++);
                laneAppr = leg.InsertApproachLane(legItemPosition++);

                // Change the Roundabout data on the Leg
                ISIAPILeg_roundabout legRou = leg.Leg_roundabout;
                legRou.Num_circulating_lanes = 2;
                legRou.Circulating_width = 10;
                legRou.Island_diameter = 36;

                // Add a Leg on East
                leg = site.AddLeg(2);
                leg.Name = "Park Road";
                legItemPosition = 1;

                // Add two Exit Lanes
                leg.InsertExitLane(legItemPosition++);
                leg.InsertExitLane(legItemPosition++);

                // Add a Roundabout Splitter Island
                leg.InsertIsland_RoundaboutSplitter(legItemPosition++);

                // Add two Approach Lanes
                laneAppr = leg.InsertApproachLane(legItemPosition++);
                laneAppr = leg.InsertApproachLane(legItemPosition++);

                // Change the Roundabout data on the Leg
                legRou = leg.Leg_roundabout;
                legRou.Num_circulating_lanes = 3;
                legRou.Circulating_width = 12;
                legRou.Island_diameter = 36;

                // Add a Leg on North
                leg = site.AddLeg(4);
                leg.Name = "Eley Road";
                legItemPosition = 1;

                // Add three Exit Lanes
                leg.InsertExitLane(legItemPosition++);
                leg.InsertExitLane(legItemPosition++);
                leg.InsertExitLane(legItemPosition++);

                leg.InsertIsland_RoundaboutSplitter(legItemPosition++);

                // Add three Approach Lanes
                laneAppr = leg.InsertApproachLane(legItemPosition++);
                laneAppr = leg.InsertApproachLane(legItemPosition++);
                laneAppr = leg.InsertApproachLane(legItemPosition++);

                // Change the Roundabout data on the Leg
                legRou = leg.Leg_roundabout;
                legRou.Num_circulating_lanes = 2;
                legRou.Circulating_width = 10;
                legRou.Island_diameter = 36;

                // Add a Leg on West
                leg = site.AddLeg(6);
                leg.Name = "Park Road";
                legItemPosition = 1;

                // Add two Exit Lanes
                leg.InsertExitLane(legItemPosition++);
                leg.InsertExitLane(legItemPosition++);

                leg.InsertIsland_RoundaboutSplitter(legItemPosition++);

                // Add two Approach Lanes
                laneAppr = leg.InsertApproachLane(legItemPosition++);
                laneAppr = leg.InsertApproachLane(legItemPosition++);

                // Change the Roundabout data on the Leg
                legRou = leg.Leg_roundabout;
                legRou.Num_circulating_lanes = 3;
                legRou.Circulating_width = 12;
                legRou.Island_diameter = 36;

                // Set the default Lane Disciplines on all Approach Lanes on this Site
                SetDefaultLaneDisciplines(site);

                // Customise the Lane Disciplines here if required...
                // after that, call site->UpdateGeometryData()

                Console.WriteLine("Updating Geometry Data...");

                // Call the method UpdateGeometryData() after Leg(s) and Lane(s) are added or removed.
                site.UpdateGeometryData();

                Console.WriteLine("Updated Geometry Data.");

                // Set Vehicle Origin-Destination (OD) Movements data, e.g. Volumes
                ISIAPIMovement_vehicle_od movVehOD = site.MovementVehicleODs[0, 6];  // Vehicle OD Movement South to West
                movVehOD.MovementVehicleODMCs[1].Volume = 25;
                movVehOD.MovementVehicleODMCs[2].Volume = 1;

                movVehOD = site.MovementVehicleODs[0, 4];
                movVehOD.MovementVehicleODMCs[1].Volume = 150;
                movVehOD.MovementVehicleODMCs[2].Volume = 3;

                movVehOD = site.MovementVehicleODs[0, 2];
                movVehOD.MovementVehicleODMCs[1].Volume = 100;
                movVehOD.MovementVehicleODMCs[2].Volume = 2;

                movVehOD = site.MovementVehicleODs[2, 0];
                movVehOD.MovementVehicleODMCs[1].Volume = 25;
                movVehOD.MovementVehicleODMCs[2].Volume = 1;

                movVehOD = site.MovementVehicleODs[2, 6];
                movVehOD.MovementVehicleODMCs[1].Volume = 150;
                movVehOD.MovementVehicleODMCs[2].Volume = 3;

                movVehOD = site.MovementVehicleODs[2, 4];
                movVehOD.MovementVehicleODMCs[1].Volume = 100;
                movVehOD.MovementVehicleODMCs[2].Volume = 2;

                movVehOD = site.MovementVehicleODs[4, 2];
                movVehOD.MovementVehicleODMCs[1].Volume = 25;
                movVehOD.MovementVehicleODMCs[2].Volume = 1;

                movVehOD = site.MovementVehicleODs[4, 0];
                movVehOD.MovementVehicleODMCs[1].Volume = 150;
                movVehOD.MovementVehicleODMCs[2].Volume = 3;

                movVehOD = site.MovementVehicleODs[4, 6];
                movVehOD.MovementVehicleODMCs[1].Volume = 100;
                movVehOD.MovementVehicleODMCs[2].Volume = 2;

                movVehOD = site.MovementVehicleODs[6, 4];
                movVehOD.MovementVehicleODMCs[1].Volume = 25;
                movVehOD.MovementVehicleODMCs[2].Volume = 1;

                movVehOD = site.MovementVehicleODs[6, 2];
                movVehOD.MovementVehicleODMCs[1].Volume = 150;
                movVehOD.MovementVehicleODMCs[2].Volume = 3;

                movVehOD = site.MovementVehicleODs[6, 0];
                movVehOD.MovementVehicleODMCs[1].Volume = 100;
                movVehOD.MovementVehicleODMCs[2].Volume = 2;

                // Set Pedestrian Movement Data

                // Staged Crossing (Type: 3), South Leg (Origin: 0), Stage 1 (Approach Side)
                ISIAPIMovement_ped movPed = site.MovementPeds[3, 0, 1];
                movPed.Exists = true;
                movPed.Volume = 25;

                // Staged Crossing (Type: 3), South Leg (Origin: 0), Stage 2 (Exit Side)
                movPed = site.MovementPeds[3, 0, 2];
                movPed.Exists = true;
                movPed.Volume = 25;

                // Staged Crossing (Type: 3), East Leg (Origin: 2), Stage 1 (Approach Side)
                movPed = site.MovementPeds[3, 2, 1];
                movPed.Exists = true;
                movPed.Volume = 25;

                // Staged Crossing (Type: 3), East Leg (Origin: 2), Stage 2 (Exit Side)
                movPed = site.MovementPeds[3, 2, 2];
                movPed.Exists = true;
                movPed.Volume = 25;

                movPed = site.MovementPeds[3, 4, 1];
                movPed.Exists = true;
                movPed.Volume = 25;
                movPed = site.MovementPeds[3, 4, 2];
                movPed.Exists = true;
                movPed.Volume = 25;

                movPed = site.MovementPeds[3, 6, 1];
                movPed.Exists = true;
                movPed.Volume = 25;
                movPed = site.MovementPeds[3, 6, 2];
                movPed.Exists = true;
                movPed.Volume = 25;

                Console.WriteLine("Processing site...");
                if (site.Process())
                {
                    var time = site.Outputset.Generated;
                    Console.WriteLine("Processed site at {0}.", time);
                }
                else
                {
                    Console.WriteLine(site.ProcessingError);
                }

                Console.WriteLine("\nFinished successfully.");
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                if (ex.InnerException != null)
                    Console.WriteLine(ex.InnerException.Message);
            }
            finally
            {
                if (_api != null)
                    _api.Close();
            }
        }

        /// <summary>
        /// // Set the default Lane Disciplines on all Approach Lanes of a Site
        /// </summary>
        static void SetDefaultLaneDisciplines(ISIAPISite site)
        {
            List<int> destinations = new List<int>();

            for (int orientation = 0; orientation < MAXOrientation; orientation++)
            {
                var leg = site.Legs[orientation];
                if (leg != null && (leg.LegGeometry == 1 || leg.LegGeometry == 3))  // Two-Way or One-Way Exit
                {
                    destinations.Add(orientation);
                }
            }

            //Console.WriteLine("Destinations:");
            //destinations.ForEach(d => Console.WriteLine(d));

            for (int orientation = 0; orientation < MAXOrientation; orientation++)
            {
                var leg = site.Legs[orientation];
                if (leg != null && (leg.LegGeometry == 1 || leg.LegGeometry == 2))  // Two-Way or One-Way Approach
                {
                    var possibleDests = destinations.Except(new int[] { leg.Orientation }).ToList(); // skip U-Turn

                    SetLegDefaultLaneDisciplines(site, leg, possibleDests);
                }
            }
        }

        static void SetLegDefaultLaneDisciplines(ISIAPISite site, ISIAPILeg leg, List<int> possibleDests)
        {
            int laneApprsCount = 0;

            var laneApprs = leg.LaneApproachs;
            laneApprsCount = laneApprs.Count;

            if (laneApprsCount == 0)
                return;
            else if (laneApprsCount == 1)
            {
                SetLaneDisciplines(laneApprs[1], possibleDests);
            }
            else
            {
                int origin = leg.Orientation;

                List<int> leftTurnDests = new List<int>();
                List<int> thruDests = new List<int>();
                List<int> rightTurnDests = new List<int>();

                foreach (int dest in possibleDests)
                {
                    int localTurn = ConvertODToLocalTurn(origin, dest);
                    switch (localTurn)
                    {
                        case 0:
                            if (site.Driveonleft)
                                rightTurnDests.Add(dest);
                            else
                                leftTurnDests.Add(dest);
                            break;
                        case 1:
                        case 2:
                        case 3:
                            leftTurnDests.Add(dest);
                            break;
                        case 4:
                            thruDests.Add(dest);
                            break;
                        default:
                            rightTurnDests.Add(dest);
                            break;
                    }
                }

                if (leftTurnDests.Count < 1) // no left turn, try add a thru or right turn
                {
                    int[] increments = new int[] { 4, 3, 2, 1 };

                    foreach (int incr in increments)
                    {
                        int tempDest = (origin + incr) % MAXOrientation;

                        if (possibleDests.Contains(tempDest))
                        {
                            leftTurnDests.Add(tempDest);
                            break;
                        }
                    }
                }

                if (thruDests.Count < 1) // no thru movement
                {
                    int[] increments = site.Driveonleft ?
                        new int[] { 5, 3, 6, 2, 7, 1 } : // Drive-on-left, Find the first possible destination in the order of L1, R1, L2, R2, L3, R3, U
                        new int[] { 3, 5, 2, 6, 1, 7 };  // Drive-on-right, Find the first possible destination in the order of R1, L1, R2, L2, R3, L3, U

                    foreach (int incr in increments)
                    {
                        int tempDest = (origin + incr) % MAXOrientation;

                        if (possibleDests.Contains(tempDest))
                        {
                            thruDests.Add(tempDest);
                            break;
                        }
                    }
                }

                if (rightTurnDests.Count < 1) // no right turn, try add a thru or left turn
                {
                    int[] increments = new int[] { 4, 5, 6, 7 };

                    foreach (int incr in increments)
                    {
                        int tempDest = (origin + incr) % MAXOrientation;

                        if (possibleDests.Contains(tempDest))
                        {
                            rightTurnDests.Add(tempDest);
                            break;
                        }
                    }
                }

                // Set the Lane Disciplines on Approach Lanes
                for (int laneno = 1; laneno <= laneApprsCount; laneno++)
                {
                    var laneAppr = laneApprs[laneno];

                    laneAppr.RemoveDisciplines(); // Clear all Disciplines on this Approach Lane

                    //Console.WriteLine("Laneno: {0}", laneno);

                    if (laneno == 1)
                    {
                        // set the first Approach Lane Disciplines
                        SetLaneDisciplines(laneAppr, leftTurnDests);
                        //Console.WriteLine("Set the first Lane {0} Disciplines", laneno);
                    }
                    else if (laneno == laneApprsCount)
                    {
                        // set the last Approach Lane Disciplines
                        SetLaneDisciplines(laneAppr, rightTurnDests);
                        //Console.WriteLine("Set the last Lane {0} Disciplines", laneno);
                    }
                    else
                    {
                        SetLaneDisciplines(laneAppr, thruDests);
                        //Console.WriteLine("Set a middle Lane {0} Disciplines", laneno);
                    }

                    if (laneno == 1 || laneno == laneApprsCount)
                    {
                        // if the first or last Approach Lane is Full-Length Lane, add the through destination as well
                        if (laneAppr.Configuration == 1) // Full-Length Lane
                        {
                            SetLaneDisciplines(laneAppr, thruDests);
                        }
                    }
                }
            }
        }

        static void SetLaneDisciplines(ISIAPILaneApproach laneAppr, List<int> destinations)
        {
            var laneApprMovs = laneAppr.LaneApproachMovements;
            foreach (int dest in destinations)
            {
                var laneApprMovMCs = laneApprMovs[dest].LaneApproachMovementMCs;
                laneApprMovMCs[1].Exists = true;
                laneApprMovMCs[2].Exists = true;
            }
        }

        static int ConvertODToLocalTurn(int origin, int destination)
        {
            int localTurn = (origin - destination) % MAXOrientation;

            if (localTurn < 0)
                localTurn += 8;

            // if localTurn == 0, it is a U-Turn movement
            // if localTurn > 0 && localTurn < 4, it is a Left-Turn Movement
            // if localTurn == 4, it is a Thru Movement
            // if localTurn > 4 && localTurn < 8, it is a Right-Turn Movement

            return localTurn;
        }

        /// <summary>
        /// A sample to read Lane Disciplines data
        /// </summary>
        static void ReadLaneDisciplines()
        {
            try
            {
                _api = new SIAPI();

                _api.OpenProject(@"C:\Temp\APITest.sip8");

                ISIAPIProject project = _api.Project;
                ISIAPISites sites = project.Sites;

                for (int siteIndex = 0; siteIndex < sites.Count; siteIndex++)
                {
                    var site = sites[siteIndex];
                    if (site == null)
                        continue;

                    Console.WriteLine("Reading Site '{0}':", site.Name);

                    bool[] includedMCs = readIncludedMCs(site);

                    //Console.WriteLine("Movement Classes:");
                    //for (int mcClass = 1; mcClass <= MAXMcClass; mcClass++)
                    //    Console.WriteLine("Movement Class [{0}] included = {1}", mcClass, includedMCs[mcClass - 1]);

                    bool[,] existingVehODs = readExistingVehODs(site);

                    //Console.WriteLine();
                    //Console.WriteLine("Movement Veh ODs:");
                    //for (int origin = 0; origin < MAXOrientation; origin++)
                    //    for (int dest = 0; dest < MAXOrientation; dest++)
                    //        Console.WriteLine("Movement Veh OD [{0}, {1}] eixsting = {2}", origin, dest, existingVehODs[origin, dest]);

                    var legs = site.Legs;
                    for (int origin = 0; origin < MAXOrientation; origin++)
                    {
                        var leg = legs[origin];
                        if (leg == null)
                            continue;

                        var laneApprs = leg.LaneApproachs;
                        for (int laneno = 1; laneno <= laneApprs.Count; laneno++)
                        {
                            var laneAppr = laneApprs[laneno];
                            if (laneAppr != null)
                            {
                                Console.WriteLine("{0}Lane Disciplines on Leg {1}, Approach Lane {2}:", Environment.NewLine, origin, laneno);
                                bool[,] laneApprMovMCExists = readLaneApprLaneDisciplines(laneAppr, origin, existingVehODs, includedMCs);
                            }
                        }
                    }
                    Console.WriteLine();
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                if (ex.InnerException != null)
                    Console.WriteLine(ex.InnerException.Message);
            }
            finally
            {
                if (_api != null)
                    _api.Close();
            }
        }

        static bool[,] readLaneApprLaneDisciplines(ISIAPILaneApproach laneAppr, int origin, bool[,] existingVehODs, bool[] includedMCs)
        {
            bool[,] laneApprMovMCExists = new bool[MAXOrientation, MAXMcClass];

            var laneApprMovs = laneAppr.LaneApproachMovements;  // get the Lane Approach Movements 'laneApprMovs' on this Lane, i.e. the Lane Disciplines
            for (int dest = 0; dest < MAXOrientation; dest++)
            {
                if (existingVehODs[origin, dest])  // make sure that the Veh OD Movement is existing
                {
                    var laneApprMov = laneApprMovs[dest];  // get the Lane Approach Movement to the particular destination
                    if (laneApprMov != null)
                    {
                        var laneApprMovMCs = laneApprMov.LaneApproachMovementMCs;
                        for (int mcClass = 1; mcClass <= MAXMcClass; mcClass++)
                        {
                            if (includedMCs[mcClass - 1]) // make sure that the Movement Classes is included
                            {
                                var laneApprMovMc = laneApprMovMCs[mcClass]; // get the Lane Approach Movement for the particular Movement Class, i.e. the Lane Discipline for this Movement Class
                                laneApprMovMCExists[dest, mcClass - 1] = laneApprMovMc == null ? false : laneApprMovMc.Exists;  // if laneApprMovMc.Exists is true, this Movement Class is allocated to go from this Approach Lane to the dest Leg

                                if (laneApprMovMCExists[dest, mcClass - 1])
                                    Console.WriteLine("Lane Approach Mov MC [{0}, {1}] exists", dest, mcClass);
                            }
                        }
                    }
                }
            }

            return laneApprMovMCExists;
        }

        /// <summary>
        /// Find the included Movement Classes in this Site
        /// </summary>
        static bool[] readIncludedMCs(ISIAPISite site)
        {
            bool[] includedMCs = new bool[MAXMcClass];

            var movClasses = site.MovementClasses;
            for (int mcClass = 1; mcClass <= MAXMcClass; mcClass++)
            {
                var movClass = movClasses[mcClass];

                includedMCs[mcClass - 1] = movClass == null ? false : movClass.Is_included;
            }

            return includedMCs;
        }

        /// <summary>
        /// Find the existing Veh ODs in this Site
        /// </summary>
        static bool[,] readExistingVehODs(ISIAPISite site)
        {
            bool[,] existingVehODs = new bool[MAXOrientation, MAXOrientation];

            var movVehODs = site.MovementVehicleODs;

            for (int origin = 0; origin < MAXOrientation; origin++)
                for (int dest = 0; dest < MAXOrientation; dest++)
                {
                    var movVehOD = movVehODs[origin, dest];
                    existingVehODs[origin, dest] = movVehOD == null ? false : movVehOD.Exists;
                }

            return existingVehODs;
        }

        /// <summary>
        /// Find the existing Peds in this Site
        /// </summary>
        static void readExistingPeds(ISIAPISite site, out bool existingPedDiagonal, out bool[,] existingPeds)
        {
            existingPeds = new bool[MAXOrientation, MAXPedCode];

            var movPeds = site.MovementPeds;

            var movPedDiagonal = movPeds[4, 0, 0];  // Ped Mov Type 4: Diagonal Pedestrian Movement
            existingPedDiagonal = movPedDiagonal == null ? false : movPedDiagonal.Exists;

            for (int origin = 0; origin < MAXOrientation; origin++)
                for (int pedCode = 0; pedCode < MAXPedCode; pedCode++)
                {
                    int pedType, pedStage;

                    if (convertPedCodeToTypeStage(pedCode, out pedType, out pedStage))
                    {
                        var movPed = movPeds[pedType, origin, pedStage];
                        existingPeds[origin, pedCode] = movPed == null ? false : movPed.Exists;
                    }
                }
        }

        /// <summary>
        /// Convert Ped Code (in array dimension) to Ped Type and Ped Stage
        /// </summary>
        static bool convertPedCodeToTypeStage(int pedCode, out int pedType, out int pedStage)
        {
            switch (pedCode)
            {
                case 0:  // Full Crossing
                    pedType = 1; // Ped Mov Type 1: Pedestrian Full Crossing Movement
                    pedStage = 0;
                    break;
                case 1:  // Slip Lane Crossing
                    pedType = 2; // Ped Mov Type 2: Pedestrian Slip Lane Crossing Movement
                    pedStage = 0;
                    break;
                case 2:  // Stage1 Crossing
                    pedType = 3; // Ped Mov Type 3: Pedestrian Staged Crossing Movement
                    pedStage = 1;
                    break;
                case 3:  // Stage2 Crossing
                    pedType = 3; // Ped Mov Type 3: Pedestrian Staged Crossing Movement
                    pedStage = 2;
                    break;
                default:
                    pedType = 0;
                    pedStage = 0;
                    return false;
            }
            return true;
        }

        /// <summary>
        /// Convert Ped Type and Ped Stage to Ped Code (in array dimension)
        /// </summary>
        static bool convertPedTypeStageToCode(int pedType, int pedStage, out int pedCode)
        {
            switch (pedType)
            {
                case 1: // Ped Mov Type 1: Pedestrian Full Crossing Movement
                    pedCode = 0;
                    break;
                case 2: // Ped Mov Type 2: Pedestrian Slip Lane Crossing Movement
                    pedCode = 1;
                    break;
                case 3: // Ped Mov Type 3: Pedestrian Staged Crossing Movement
                    if (pedStage == 1)
                        pedCode = 2;
                    else if (pedStage == 2)
                        pedCode = 3;
                    else
                    {
                        pedCode = -1;
                        return false;
                    }
                    break;
                default:
                    pedCode = -1;
                    return false;
            }
            return true;
        }

        /// <summary>
        /// A sample to read Priorities data
        /// </summary>
        static void ReadPrioritiesData()
        {
            try
            {
                _api = new SIAPI();

                _api.OpenProject(@"C:\Temp\APITest.sip8");

                ISIAPIProject project = _api.Project;
                ISIAPISites sites = project.Sites;

                for (int siteIndex = 0; siteIndex < sites.Count; siteIndex++)
                {
                    var site = sites[siteIndex];
                    if (site == null)
                        continue;

                    Console.WriteLine("Reading Site '{0}':", site.Name);

                    bool[,] existingVehODs = readExistingVehODs(site);

                    bool existingPedDiagonal;
                    bool[,] existingPeds;

                    readExistingPeds(site, out existingPedDiagonal, out existingPeds);

                    bool[, , ,] opposingVehData = readOpposingVehData(site, existingVehODs);

                    bool[, , ,] opposingPedData = readOpposingPedData(site, existingVehODs, existingPeds);

                    Console.WriteLine();
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                if (ex.InnerException != null)
                    Console.WriteLine(ex.InnerException.Message);
            }
            finally
            {
                if (_api != null)
                    _api.Close();
            }
        }

        /// <summary>
        /// Read Opposing Veh OD Movement data for all Veh OD Movements in this Site
        /// </summary>
        static bool[, , ,] readOpposingVehData(ISIAPISite site, bool[,] existingVehODs)
        {
            bool[, , ,] opposingVehData = new bool[MAXOrientation, MAXOrientation, MAXOrientation, MAXOrientation];

            var movVehODs = site.MovementVehicleODs;
            for (int origin = 0; origin < MAXOrientation; origin++)
                for (int dest = 0; dest < MAXOrientation; dest++)
                {
                    if (existingVehODs[origin, dest])
                    {
                        Console.WriteLine("{0}Veh Mov ({1}, {2}) is opposed by:", Environment.NewLine, origin, dest);
                        var movVehOD = movVehODs[origin, dest];

                        var opposingMovVehs = movVehOD.Opposingmovement_vehicles;

                        for (int opposingVehIndex = 0; opposingVehIndex < opposingMovVehs.Count; opposingVehIndex++)
                        {
                            var opposingMovVeh = opposingMovVehs[opposingVehIndex];
                            int opposingOrigin = opposingMovVeh.Opposingmovement_vehicle_origin;
                            int opposingDest = opposingMovVeh.Opposingmovement_vehicle_destination;
                            if (existingVehODs[opposingOrigin, opposingDest])  // make sure the Opposing Veh Movement exists.
                            {
                                opposingVehData[origin, dest, opposingOrigin, opposingDest] = opposingMovVeh.Opposing;
                                // If opposing is true, the Vehicle Movement (opposingOrigin, opposingDest) is opposing the Vehicle Movement (origin, dest)

                                if (opposingMovVeh.Opposing)
                                    Console.WriteLine("Veh Mov ({0}, {1})", opposingOrigin, opposingDest);
                            }
                        }
                    }
                }

            return opposingVehData;
        }

        /// <summary>
        /// Read Opposing Ped Movement data for all Veh OD Movements in this Site
        /// </summary>
        static bool[, , ,] readOpposingPedData(ISIAPISite site, bool[,] existingVehODs, bool[,] existingPeds)
        {
            bool[, , ,] opposingPedData = new bool[MAXOrientation, MAXOrientation, MAXOrientation, MAXPedCode];

            var movVehODs = site.MovementVehicleODs;
            for (int origin = 0; origin < MAXOrientation; origin++)
                for (int dest = 0; dest < MAXOrientation; dest++)
                {
                    if (existingVehODs[origin, dest])
                    {
                        Console.WriteLine("{0}Veh Mov ({1}, {2}) is opposed by:", Environment.NewLine, origin, dest);
                        var movVehOD = movVehODs[origin, dest];

                        var opposingMovPeds = movVehOD.Opposingmovement_peds;

                        for (int opposingPedIndex = 0; opposingPedIndex < opposingMovPeds.Count; opposingPedIndex++)
                        {
                            var opposingMovPed = opposingMovPeds[opposingPedIndex];

                            int opposingPedType = opposingMovPed.Opposingmovement_ped_type;
                            int opposingPedOrigin = opposingMovPed.Opposingmovement_ped_origin;
                            int opposingPedStage = opposingMovPed.Opposingmovement_ped_stage;
                            int opposingPedCode;
                            if (convertPedTypeStageToCode(opposingPedType, opposingPedStage, out opposingPedCode))
                            {
                                if (existingPeds[opposingPedOrigin, opposingPedCode]) // make sure the Opposing Ped Movement exists.
                                {
                                    opposingPedData[origin, dest, opposingPedOrigin, opposingPedCode] = opposingMovPed.Opposing;
                                    // If opposing is true, the Ped Movement (opposingPedOrigin, opposingPedCode) is opposing the Vehicle Movement (origin, dest)

                                    if (opposingMovPed.Opposing)
                                        Console.WriteLine("Ped Mov (Type:{0}, Origin:{1}, Stage:{2})", opposingPedType, opposingPedOrigin, opposingPedStage);
                                }
                            }
                        }
                    }
                }

            return opposingPedData;
        }
    }
}
