#include "stdafx.h"
#include "tchar.h"

#include "CppSIAPI.h"

#include <iostream>
#include <string>
#include <list>
#include <time.h>
#include <ATLComTime.h>

#include <atlcomcli.h>

using namespace SIDRASolutions_SI_API;

namespace CppSample
{
	CppSIAPI::CppSIAPI()
	{
		// Initialize COM.
		CoInitialize(NULL);

		pISIAPI.CoCreateInstance(__uuidof(SIAPI));
	}


	CppSIAPI::~CppSIAPI()
	{
		if (pISIAPI != NULL)
		{
			VARIANT_BOOL boolResult = false;
			pISIAPI->Close(&boolResult);
			CoUninitialize();
		}
	}

	CComPtr<ISIAPI> CppSIAPI::get_SIAPI()
	{
		return pISIAPI;
	}

	// Find the included Movement Classes in this Site
	int CppSIAPI::ReadIncludedMCs(CComPtr<ISIAPISite> site, bool includedMCs[MAXMcClass])
	{
		CComPtr<ISIAPIMovementClasses> movClasses = NULL;
		CComPtr<ISIAPIMovementClass> movClass = NULL;
		long mcClass;
		VARIANT_BOOL included = false;

		site->get_MovementClasses(&movClasses);
		for (mcClass = 1; mcClass <= MAXMcClass; mcClass++)
		{
			movClasses->get_Item(mcClass, &movClass); // the parameter mcClass is a one-base integer
			if (movClass == NULL)
				includedMCs[mcClass - 1] = false;
			else
			{
				movClass->get_Is_included(&included);
				if (included)
					includedMCs[mcClass - 1] = true;
				else
					includedMCs[mcClass - 1] = false;
			}
		}

		return 0;
	}

	// Find the existing Veh ODs in this Site
	int CppSIAPI::ReadExistingVehODs(CComPtr<ISIAPISite> site, bool existingVehODs[MAXOrientation][MAXOrientation])
	{
		CComPtr<ISIAPIMovement_vehicle_ods> movVehODs = NULL;
		CComPtr<ISIAPIMovement_vehicle_od> movVehOD = NULL;
		long origin, dest;
		VARIANT_BOOL existing = false;

		site->get_MovementVehicleODs(&movVehODs);

		for (origin = 0; origin < MAXOrientation; origin++)
			for (dest = 0; dest < MAXOrientation; dest++)
			{
				movVehODs->get_Item(origin, dest, &movVehOD);
				if (movVehOD == NULL)
					existingVehODs[origin][dest] = false;
				else
				{
					movVehOD->get_Exists(&existing);
					if (existing)
						existingVehODs[origin][dest] = true;
					else
						existingVehODs[origin][dest] = false;
				}
			}

		return 0;
	}

	// Find the existing Peds in this Site
	int CppSIAPI::ReadExistingPeds(CComPtr<ISIAPISite> site, bool *existingPedDiagonal, bool existingPeds[MAXOrientation][MAXPedCode])
	{
		CComPtr<ISIAPIMovement_peds> movPeds = NULL;
		CComPtr<ISIAPIMovement_ped> movPed = NULL;
		long origin, pedCode, pedType, pedStage;
		VARIANT_BOOL existing = false;

		site->get_MovementPeds(&movPeds);

		movPeds->get_Item(4, 0, 0, &movPed); // Ped Mov Type 4: Diagonal Pedestrian Movement
		if (movPed == NULL)
			*existingPedDiagonal = false;
		else
		{
			movPed->get_Exists(&existing);
			if (existing)
				*existingPedDiagonal = true;
			else
				*existingPedDiagonal = false;
		}

		for (origin = 0; origin < MAXOrientation; origin++)
			for (pedCode = 0; pedCode < MAXPedCode; pedCode++)
			{
				existing = false;
				if (ConvertPedCodeToTypeStage(pedCode, &pedType, &pedStage) == 0)
				{
					movPeds->get_Item(pedType, origin, pedStage, &movPed);
					if (movPed != NULL)
						movPed->get_Exists(&existing);
				}

				if (existing)
					existingPeds[origin][pedCode] = true;
				else
					existingPeds[origin][pedCode] = false;
			}

		return 0;
	}

	// Convert Ped Code (in array dimension) to Ped Type and Ped Stage
	int CppSIAPI::ConvertPedCodeToTypeStage(long pedCode, long *pedType, long *pedStage)
	{
		switch (pedCode)
		{
		case 0:  // Full Crossing
			*pedType = 1; // Ped Mov Type 1: Pedestrian Full Crossing Movement
			*pedStage = 0;
			break;
		case 1:  // Slip Lane Crossing
			*pedType = 2; // Ped Mov Type 2: Pedestrian Slip Lane Crossing Movement
			*pedStage = 0;
			break;
		case 2:  // Stage1 Crossing
			*pedType = 3; // Ped Mov Type 3: Pedestrian Staged Crossing Movement
			*pedStage = 1;
			break;
		case 3:  // Stage2 Crossing
			*pedType = 3; // Ped Mov Type 3: Pedestrian Staged Crossing Movement
			*pedStage = 2;
			break;
		default:
			return 1;
		}

		return 0;
	}

	// Convert Ped Type and Ped Stage to Ped Code (in array dimension) 
	int CppSIAPI::ConvertPedTypeStageToCode(long pedType, long pedStage, long *pedCode)
	{
		switch (pedType)
		{
		case 1: // Ped Mov Type 1: Pedestrian Full Crossing Movement
			*pedCode = 0;
			break;
		case 2: // Ped Mov Type 2: Pedestrian Slip Lane Crossing Movement
			*pedCode = 1;
			break;
		case 3: // Ped Mov Type 3: Pedestrian Staged Crossing Movement
			if (pedStage == 1)
				*pedCode = 2;
			else if (pedStage == 2)
				*pedCode = 3;
			else
				return 1;
			break;
		default:
			return 1;
		}
		return 0;
	}

	// Read Lane Disciplines on an Approach Lane
	int CppSIAPI::ReadLaneApprLaneDisciplines(CComPtr<ISIAPILaneApproach> laneAppr, long origin,
		bool existingVehODs[MAXOrientation][MAXOrientation], bool includedMCs[MAXMcClass],
		bool laneApprMovMCExists[MAXOrientation][MAXMcClass])
	{
		long dest, mcClass;
		VARIANT_BOOL laneDisciplineExisting = false;

		CComPtr<ISIAPILeg> leg = NULL;
		CComPtr<ISIAPILaneApproachMovements> laneApprMovs = NULL;
		CComPtr<ISIAPILaneApproachMovement> laneApprMov = NULL;
		CComPtr<ISIAPILaneApproachMovementMCs> laneApprMovMCs = NULL;
		CComPtr<ISIAPILaneApproachMovementMC> laneApprMovMC = NULL;

		laneAppr->get_LaneApproachMovements(&laneApprMovs);  // get the Lane Approach Movements 'laneApprMovs' on this Lane, i.e. the Lane Disciplines
		for (dest = 0; dest < MAXOrientation; dest++)
		{
			// clear data
			for (mcClass = 1; mcClass <= MAXMcClass; mcClass++)
			{
				laneApprMovMCExists[dest][mcClass - 1] = false;
			}
			
			if (existingVehODs[origin][dest]) // make sure that the Veh OD Movement is existing
			{
				laneApprMovs->get_Item(dest, &laneApprMov);  // get the Lane Approach Movement to the particular destination
				if (laneApprMov != NULL)
				{
					laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
					for (mcClass = 1; mcClass <= MAXMcClass; mcClass++)
					{
						if (includedMCs[mcClass - 1]) // make sure that the Movement Classes is included
						{
							laneApprMovMCs->get_Item(mcClass, &laneApprMovMC); // get the Lane Approach Movement for the particular Movement Class, i.e. the Lane Discipline for this Movement Class
							if (laneApprMovMC != NULL)
							{
								laneApprMovMC->get_Exists(&laneDisciplineExisting);  // if laneDisciplineExisting is true, this Movement Class is allocated to go from this Approach Lane to the dest Leg
								if (laneDisciplineExisting)
									laneApprMovMCExists[dest][mcClass - 1] = true;
							}
						}
					}
				}
			}
		}

		return 0;
	}

	// Read Opposing Veh OD Movement data for all Veh OD Movements in this Site
	int CppSIAPI::ReadOpposingVehData(CComPtr<ISIAPISite> site,
		bool existingVehODs[MAXOrientation][MAXOrientation],
		bool opposingVehData[MAXOrientation][MAXOrientation][MAXOrientation][MAXOrientation])
	{
		long origin, dest, opposingVehCount, opposingVehIndex, opposingOrigin, opposingDest;
		CComPtr<ISIAPIMovement_vehicle_ods> movVehODs = NULL;
		CComPtr<ISIAPIMovement_vehicle_od> movVehOD = NULL;
		CComPtr<ISIAPIOpposingmovement_vehicles> opposingMovVehs = NULL;
		CComPtr<ISIAPIOpposingmovement_vehicle> opposingMovVeh = NULL;

		VARIANT_BOOL opposing = false;

		site->get_MovementVehicleODs(&movVehODs);

		for (origin = 0; origin < MAXOrientation; origin++)
		{
			for (dest = 0; dest < MAXOrientation; dest++)
			{
				// clear data
				for (opposingOrigin = 0; opposingOrigin < MAXOrientation; opposingOrigin++)
				{
					for (opposingDest = 0; opposingDest < MAXOrientation; opposingDest++)
					{
						opposingVehData[origin][dest][opposingOrigin][opposingDest] = false;
					}
				}

				// read data
				if (existingVehODs[origin][dest])
				{
					movVehODs->get_Item(origin, dest, &movVehOD);  // movVehOD is the Opposed Movement

					movVehOD->get_Opposingmovement_vehicles(&opposingMovVehs);  // opposingMovVehs are the Veh Movements that can be set to oppose the movVehOD
					opposingMovVehs->get_Count(&opposingVehCount);

					for (opposingVehIndex = 0; opposingVehIndex < opposingVehCount; opposingVehIndex++)
					{
						opposingMovVehs->get_Item_2(opposingVehIndex, &opposingMovVeh);
						opposingMovVeh->get_Opposingmovement_vehicle_origin(&opposingOrigin);
						opposingMovVeh->get_Opposingmovement_vehicle_destination(&opposingDest);

						if (existingVehODs[opposingOrigin][opposingDest]) // make sure the Opposing Veh Movement exists.
						{
							opposingMovVeh->get_Opposing(&opposing);
							if (opposing) // If opposing is true, the Vehicle Movement (opposingOrigin, opposingDest) is opposing the Vehicle Movement (origin, dest)
								opposingVehData[origin][dest][opposingOrigin][opposingDest] = true;
						}
					}

				}
			}
		}

		return 0;
	}

	// Read Opposing Ped Movement data for all Veh OD Movements in this Site
	int CppSIAPI::ReadOpposingPedData(CComPtr<ISIAPISite> site,
		bool existingVehODs[MAXOrientation][MAXOrientation], bool existingPeds[MAXOrientation][MAXPedCode],
		bool opposingPedData[MAXOrientation][MAXOrientation][MAXOrientation][MAXPedCode])
	{
		long origin, dest, opposingPedCount, opposingPedIndex, opposingPedOrigin, opposingPedCode, opposingPedType, opposingPedStage;
		CComPtr<ISIAPIMovement_vehicle_ods> movVehODs = NULL;
		CComPtr<ISIAPIMovement_vehicle_od> movVehOD = NULL;
		CComPtr<ISIAPIOpposingmovement_peds> opposingMovPeds = NULL;
		CComPtr<ISIAPIOpposingmovement_ped> opposingMovPed = NULL;

		VARIANT_BOOL opposing = false;

		site->get_MovementVehicleODs(&movVehODs);

		for (origin = 0; origin < MAXOrientation; origin++)
		{
			for (dest = 0; dest < MAXOrientation; dest++)
			{
				// clear data
				for (opposingPedOrigin = 0; opposingPedOrigin < MAXOrientation; opposingPedOrigin++)
				{
					for (opposingPedCode = 0; opposingPedCode < MAXPedCode; opposingPedCode++)
					{
						opposingPedData[origin][dest][opposingPedOrigin][opposingPedCode] = false;
					}
				}

				// read data
				if (existingVehODs[origin][dest])
				{
					movVehODs->get_Item(origin, dest, &movVehOD);  // movVehOD is the Opposed Movement

					movVehOD->get_Opposingmovement_peds(&opposingMovPeds);  // opposingMovPeds are the Ped Movements that can be set to oppose the movVehOD
					opposingMovPeds->get_Count(&opposingPedCount);

					for (opposingPedIndex = 0; opposingPedIndex < opposingPedCount; opposingPedIndex++)
					{
						opposingMovPeds->get_Item_2(opposingPedIndex, &opposingMovPed);
						opposingMovPed->get_Opposingmovement_ped_type(&opposingPedType);
						opposingMovPed->get_Opposingmovement_ped_origin(&opposingPedOrigin);
						opposingMovPed->get_Opposingmovement_ped_stage(&opposingPedStage);

						if (ConvertPedTypeStageToCode(opposingPedType, opposingPedStage, &opposingPedCode) == 0)
						{
							if (existingPeds[opposingPedOrigin][opposingPedCode]) // make sure the Opposing Ped Movement exists.
							{
								opposingMovPed->get_Opposing(&opposing);
								if (opposing) // If opposing is true, the Ped Movement (opposingPedOrigin, opposingPedCode) is opposing the Vehicle Movement (origin, dest)
									opposingPedData[origin][dest][opposingPedOrigin][opposingPedCode] = true;
							}
						}
					}
				}
			}
		}

		return 0;
	}

	// Set the default Lane Disciplines on all Approach Lanes of a Site
	int CppSIAPI::SetDefaultLaneDisciplines(CComPtr<ISIAPISite> site)
	{
		CComPtr<ISIAPILegs> legs;
		CComPtr<ISIAPILeg> leg;

		long orientation = 0;
		std::list<long> destinations = std::list<long>();
		long geo = 0;

		site->get_Legs(&legs);

		for (orientation = 0; orientation < MAXOrientation; orientation++)
		{
			legs->get_Item(orientation, &leg);
			if (leg != NULL)
			{
				leg->get_LegGeometry(&geo);
				if (geo == 1 || geo == 3) // Two-Way or One-Way Exit
					destinations.push_back(orientation);
			}
		}

		//wprintf(L"Destinations:\n");
		//for (long d : destinations)
		//{
		//	wprintf(L"%i\n", d);
		//}

		for (orientation = 0; orientation < MAXOrientation; orientation++)
		{
			legs->get_Item(orientation, &leg);
			if (leg != NULL)
			{
				leg->get_LegGeometry(&geo);
				if (geo == 1 || geo == 2) // Two-Way or One-Way Approach
				{
					std::list<long> possibleDests = std::list<long>();
					for (long d : destinations)
					{
						if (d != orientation)  // skip U-Turn
							possibleDests.push_back(d);
					}

					SetLegDefaultLaneDisciplines(site, leg, possibleDests);
				}
			}
		}

		return 0;
	}

	// Set the default Lane Disciplines on all Approach Lanes of a Leg
	int CppSIAPI::SetLegDefaultLaneDisciplines(CComPtr<ISIAPISite> site, CComPtr<ISIAPILeg> leg, std::list<long> possibleDests)
	{
		long origin = 0;
		long localTurn = 0;
		long tempDest = 0;

		CComPtr<ISIAPILaneApproachs> laneApprs;
		CComPtr<ISIAPILaneApproach> laneAppr;

		long laneApprsCount = 0;
		long laneno = 0; // Lane Number
		VARIANT_BOOL boolResult = false;
		long laneConfig = 0;

		leg->get_LaneApproachs(&laneApprs);
		laneApprs->get_Count(&laneApprsCount);

		if (laneApprsCount == 0)
			return 1;
		else if (laneApprsCount == 1)
		{
			laneApprs->get_Item(1l, &laneAppr);
			SetLaneDisciplines(laneAppr, possibleDests);
		}
		else
		{
			VARIANT_BOOL driveOnLeft = false;
			site->get_DriveOnLeft(&driveOnLeft);

			leg->get_Orientation(&origin);

			std::list<long> leftTurnDests = std::list<long>();
			std::list<long> thruDests = std::list<long>();
			std::list<long> rightTurnDests = std::list<long>();

			for (long dest : possibleDests)
			{
				CovertODToLocalTurn(origin, dest, &localTurn);
				switch (localTurn)
				{
				case 0:
					if (driveOnLeft)
						rightTurnDests.push_back(dest);
					else
						leftTurnDests.push_back(dest);
					break;
				case 1:
				case 2:
				case 3:
					leftTurnDests.push_back(dest);
					break;
				case 4:
					thruDests.push_back(dest);
					break;
				default:
					rightTurnDests.push_back(dest);
					break;

				}
			}

			std::list<long> increments;

			if (leftTurnDests.size() < 1) // no left turn, try add a thru or right turn
			{
				increments = { 4, 3, 2, 1 };
				bool found = false;

				for (long incr : increments)
				{
					tempDest = (origin + incr) % MAXOrientation;

					for (long d : possibleDests)
					{
						if (d == tempDest)
						{
							leftTurnDests.push_back(tempDest);
							found = true;
							break;
						}
					}

					if (found)
						break;
				}
			}

			if (thruDests.size() < 1) // no thru movement
			{
				if (driveOnLeft)
					increments = { 5, 3, 6, 2, 7, 1 }; // Drive-on-left, Find the first possible destination in the order of L1, R1, L2, R2, L3, R3
				else
					increments = { 3, 5, 2, 6, 1, 7 }; // Drive-on-right, Find the first possible destination in the order of R1, L1, R2, L2, R3, L3

				bool found = false;

				for (long incr : increments)
				{
					tempDest = (origin + incr) % MAXOrientation;

					for (long d : possibleDests)
					{
						if (d == tempDest)
						{
							thruDests.push_back(tempDest);
							found = true;
							break;
						}
					}

					if (found)
						break;
				}
			}

			if (rightTurnDests.size() < 1) // no right turn, try add a thru or left turn
			{
				increments = { 4, 5, 6, 7 };
				bool found = false;

				for (long incr : increments)
				{
					tempDest = (origin + incr) % MAXOrientation;

					for (long d : possibleDests)
					{
						if (d == tempDest)
						{
							rightTurnDests.push_back(tempDest);
							found = true;
							break;
						}
					}

					if (found)
						break;
				}
			}

			// Set the Lane Disciplines on Approach Lanes
			//for (laneno = 1; laneno <= laneApprsCount; laneno++)
			laneno = 1;
			while (laneno <= laneApprsCount)
			{
				laneApprs->get_Item(laneno, &laneAppr);

				laneAppr->RemoveDisciplines(&boolResult); // Clear all Disciplines on this Approach Lane

				//wprintf(L"Laneno: %i\n", laneno);

				if (laneno == 1)
				{
					// set the first Approach Lane Disciplines
					SetLaneDisciplines(laneAppr, leftTurnDests);
					//wprintf(L"Set the first Lane %i Disciplines\n", laneno);
				}
				else if (laneno == laneApprsCount)
				{
					// set the last Approach Lane Disciplines
					SetLaneDisciplines(laneAppr, rightTurnDests);
					//wprintf(L"Set the last Lane %i Disciplines\n", laneno);
				}
				else
				{
					SetLaneDisciplines(laneAppr, thruDests);
					//wprintf(L"Set a middle Lane %i Disciplines\n", laneno);
				}

				if (laneno == 1 || laneno == laneApprsCount)
				{
					// if the first or last Approach Lane is Full-Length Lane, add the through destination as well
					laneAppr->get_Configuration(&laneConfig);
					if (laneConfig == 1) // Full-Length Lane
					{
						SetLaneDisciplines(laneAppr, thruDests);
					}
				}

				laneno++;
			}
		}

		return 0;
	}

	int CppSIAPI::SetLaneDisciplines(CComPtr<ISIAPILaneApproach> laneAppr, std::list<long> destinations)
	{
		CComPtr<ISIAPILaneApproachMovements> laneApprMovs;
		CComPtr<ISIAPILaneApproachMovement> laneApprMov;
		CComPtr<ISIAPILaneApproachMovementMCs> laneApprMovMCs;
		CComPtr<ISIAPILaneApproachMovementMC> laneApprMovMC;

		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		for (long dest : destinations)
		{
			laneApprMovs->get_Item(dest, &laneApprMov);
			laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
			laneApprMovMCs->get_Item(1l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);
			laneApprMovMCs->get_Item(2l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);
		}

		return 0;
	}

	int CppSIAPI::CovertODToLocalTurn(long origin, long destination, long *localTurn)
	{
		*localTurn = (origin - destination) % MAXOrientation;

		if (*localTurn < 0)
			*localTurn += 8l;

		// if localTurn == 0, it is a U-Turn movement
		// if localTurn > 0 && localTurn < 4, it is a Left-Turn Movement
		// if localTurn == 4, it is a Thru Movement
		// if localTurn > 4 && localTurn < 8, it is a Right-Turn Movement

		return 0;
	}

}